from .numpy_vector import NumpyPrimalVector
from .numpy_vector import NumpyDualVector

from .dolfin_vector import DolfinPrimalVector
from .dolfin_vector import DolfinDualVector
from .dolfin_vector import RieszMap



from .dolfin_vector_set import DolfinPrimalVectorSet
from .dolfin_vector_set import DolfinDualVectorSet
from .dolfin_vector_set import RieszMapSet

from .adaptor import convert_to_moola_dual_vector


from .line_search import LineSearch

class ArmijoLineSearch(LineSearch):
    def __init__(self, ftol = 1e-4, start_stp = 1.0, stpmin = 1e-10, stpmax=10000,
                 adaptive_stp=True):
        '''
        This class implements a line search algorithm whose steps 
        satisfy the Armijo condition, i.e. they satisfies a 
        sufficient decrease condition.

        The algorithm is designed to find a step 'stp' that satisfies
        the sufficient decrease condition

               f(stp) <= f(0) + ftol*stp*f'(0).

        There is always a step which satisfies both conditions.

        If the step size drops below stpmin, the search algorithm raises Warning.

        The function arguments are:

           ftol         | a nonnegative tolerance for the sufficient decrease condition.
           start_stp    | a guess for an initial step size. 
           stpmin       | a nonnegative lower bound for the step.
           stpmax       | a nonnegative upper bound for the step. 
           adaptive_stp | dis/enables the adaptive step size algorithm.

        Returns:
          A step size that satisfies the Armijo condition

        '''

        if ftol <= 0:
            raise ValueError("ftol must be > 0")
        if start_stp <= 0:
            raise ValueError("start_stp must be > 0")
        if stpmin <= 0:
            raise ValueError("stpmin must be > 0")
        if stpmax <=0 or stpmax <= stpmin:
            raise ValueError("stpmax must be > 0 and > stpmin")
        
        self.ftol         = ftol 
        self.start_stp    = start_stp
        self.stpmin       = stpmin
        self.adaptive_stp = adaptive_stp
        self.stpmax       = stpmax

    def _test(self, f, g, f0, stp):
        ''' Tests if the Armijo condition is satisfied '''
        return f <= f0 + self.ftol*stp*g

    def _adapt(self, it):
        ''' Adapts the starting step according '''
        if it < 2:
            self.start_stp *= 2
        if it >=2:
            self.start_stp /= 2
        if self.start_stp >= self.stpmax:
            self.start_stp /=4

    def search(self, phi, phi_dphi, phi_dphi0):
        ''' Performs the line search on the function phi. 

            phi must be a function [0, oo] -> R.
            phi_dphi is ignored (you can set it to None).
            phi_dphi0 is a tuple containing the functional value and 
            directional derivative at step=0.

            The return value is a step that satisfies the Armijo condition. 
        '''
            
        stp = self.start_stp
        finit, ginit = phi_dphi0
        if ginit >= 0:
            raise Warning("The gradient is not a descent direction")
        f = finit 
        
        it = 0
        while True:
            if self._test(f, ginit, finit, stp):
                self._adapt(it)
                return stp
            elif stp < self.stpmin:
                raise Warning("The step size dropped below the minimum step size.")

            stp /= 2.0
            it += 1
            f = phi(stp)




def dcsrch(stp, f, g, ftol, gtol, xtol, task, stpmin, stpmax, isave, dsave):
    """
     Subroutine dcsrch

     This subroutine finds a step that satisfies a sufficient
     decrease condition and a curvature condition.

     Each call of the subroutine updates an interval with
     endpoints stx and sty. The interval is initially chosen
     so that it contains a minimizer of the modified function

           psi(stp) = f(stp) - f(0) - ftol*stp*f'(0).

     If psi(stp) <= 0 and f'(stp) >= 0 for some step, then the
     interval is chosen so that it contains a minimizer of f.

     The algorithm is designed to find a step that satisfies
     the sufficient decrease condition

           f(stp) <= f(0) + ftol*stp*f'(0),

     and the curvature condition

           abs(f'(stp)) <= gtol*abs(f'(0)).

     If ftol is less than gtol and if, for example, the function
     is bounded below, then there is always a step which satisfies
     both conditions.

     If no step can be found that satisfies both conditions, then
     the algorithm stops with a warning. In this case stp only
     satisfies the sufficient decrease condition.

     A typical invocation of dcsrch has the following outline:

     Evaluate the function at stp = 0.0d0; store in f.
     Evaluate the gradient at stp = 0.0d0; store in g.
     Choose a starting step stp.

     task = 'START'
  10 continue
        call dcsrch(stp,f,g,ftol,gtol,xtol,task,stpmin,stpmax,
    +               isave,dsave)
        if (task == 'FG') then
           Evaluate the function and the gradient at stp
           go to 10
           end if

     NOTE: The user must not alter work arrays between calls.

     The subroutine statement is

       subroutine dcsrch(f,g,stp,ftol,gtol,xtol,stpmin,stpmax,
                         task,isave,dsave)
     where

       stp is a double precision variable.
         On entry stp is the current estimate of a satisfactory
            step. On initial entry, a positive initial estimate
            must be provided.
         On exit stp is the current estimate of a satisfactory step
            if task = 'FG'. If task = 'CONV' then stp satisfies
            the sufficient decrease and curvature condition.

       f is a double precision variable.
         On initial entry f is the value of the function at 0.
            On subsequent entries f is the value of the
            function at stp.
         On exit f is the value of the function at stp.

       g is a double precision variable.
         On initial entry g is the derivative of the function at 0.
            On subsequent entries g is the derivative of the
            function at stp.
         On exit g is the derivative of the function at stp.

       ftol is a double precision variable.
         On entry ftol specifies a nonnegative tolerance for the
            sufficient decrease condition.
         On exit ftol is unchanged.

       gtol is a double precision variable.
         On entry gtol specifies a nonnegative tolerance for the
            curvature condition.
         On exit gtol is unchanged.

       xtol is a double precision variable.
         On entry xtol specifies a nonnegative relative tolerance
            for an acceptable step. The subroutine exits with a
            warning if the relative difference between sty and stx
            is less than xtol.
         On exit xtol is unchanged.

       task is a character variable of length at least 60.
         On initial entry task must be set to 'START'.
         On exit task indicates the required action:

            If task(1:2) = 'FG' then evaluate the function and
            derivative at stp and call dcsrch again.

            If task(1:4) = 'CONV' then the search is successful.

            If task(1:4) = 'WARN' then the subroutine is not able
            to satisfy the convergence conditions. The exit value of
            stp contains the best point found during the search.

            If task(1:5) = 'ERROR' then there is an error in the
            input arguments.

         On exit with convergence, a warning or an error, the
            variable task contains additional information.

       stpmin is a double precision variable.
         On entry stpmin is a nonnegative lower bound for the step.
         On exit stpmin is unchanged.

       stpmax is a double precision variable.
         On entry stpmax is a nonnegative upper bound for the step.
         On exit stpmax is unchanged.

       isave is an integer work array of dimension 2.

       dsave is a double precision work array of dimension 13.

     Subprograms called

       MINPACK-2 ... dcstep

     MINPACK-1 Project. June 1983.
     Argonne National Laboratory.
     Jorge J. More' and David J. Thuente.

     MINPACK-2 Project. November 1993.
     Argonne National Laboratory and University of Minnesota.
     Brett M. Averick, Richard G. Carter, and Jorge J. More'.

     Converted and extended to Python, July 2013
     Imperial College London
     Simon W. Funke
    """

    from .dcstep import dcstep

    xtrapl=1.1
    xtrapu=4.0

    # Initialization block.
    if task == 'START':

        if stpmin is None:
          stpmin = 0
        if stpmax is None:
          stpmax = 1e999

        # Check the input arguments for errors.
        if stp < stpmin: 
            task = 'Error: stp < stpmin'
        if stp > stpmax: 
            task = 'Error: stp > stpmax'
        if g >= 0.0: 
            task = 'Error: initial g >= 0 (%e)' % g
        if ftol < 0.0: 
            task = 'Error: ftol < zero'
        if gtol < 0.0:  
            task = 'Error: gtol < zero'
        if xtol < 0.0:  
            task = 'Error: xtol < zero'
        if stpmin < 0.0: 
            task = 'Error: stpmin < zero'
        if stpmax < stpmin: 
            task = 'Error: stpmax < stpmin'

        # Exit if there are errors on input.
        if "Error" in task:
            return stp, task, isave, dsave

        # Initialize local variables.
        brackt = False
        stage = 1
        finit = f
        ginit = g
        gtest = ftol*ginit
        width = stpmax - stpmin
        width1 = width/0.5

        # The variables stx, fx, gx contain the values of the step,
        # function, and derivative at the best step.
        # The variables sty, fy, gy contain the value of the step,
        # function, and derivative at sty.
        # The variables stp, f, g contain the values of the step,
        # function, and derivative at stp.

        stx = 0.0
        fx = finit
        gx = ginit
        sty = 0.0
        fy = finit
        gy = ginit
        stmin = 0.0
        stmax = stp + xtrapu*stp
        task = 'FG'

        # Save local variables.
        isave[1] = brackt
        isave[2] = stage
        dsave[1] = ginit
        dsave[2] = gtest
        dsave[3] = gx
        dsave[4] = gy
        dsave[5] = finit
        dsave[6] = fx
        dsave[7] = fy
        dsave[8] = stx
        dsave[9] = sty
        dsave[10] = stmin
        dsave[11] = stmax
        dsave[12] = width
        dsave[13] = width1

        return stp, task, isave, dsave

    else:
        # Restore local variables.
        brackt = isave[1]
        stage = isave[2]
        ginit = dsave[1]
        gtest = dsave[2]
        gx = dsave[3]
        gy = dsave[4]
        finit = dsave[5]
        fx = dsave[6]
        fy = dsave[7]
        stx = dsave[8]
        sty = dsave[9]
        stmin = dsave[10]
        stmax = dsave[11]
        width = dsave[12]
        width1 = dsave[13]

    # If psi(stp) <= 0 and f'(stp) >= 0 for some step, then the
    # algorithm enters the second stage.
    ftest = finit + stp*gtest
    if stage == 1 and f <= ftest and g >= 0.0: 
        stage = 2

    # Test for warnings.
    if brackt and (stp <= stmin or stp >= stmax): 
        task = 'Warning: Rounding errors prevent progress'
    if brackt and stmax-stmin <= xtol*stmax:
        task = 'Warning: xtol test satisfied'
    if stp == stpmax and f <= ftest and g <= gtest:
        task = 'Warning: stp = stpmax'
    if stp == stpmin and (f > ftest or g >= gtest):
        task = 'Warning: stp = stpmin'

    # Test for convergence.
    if f <= ftest and abs(g) <= gtol*(-ginit):
        task = 'Convergence'

    # Test for termination.
    if "Warning" in task or task == 'Convergence': 
        # Save local variables.
        isave[1] = brackt
        isave[2] = stage
        dsave[1] = ginit
        dsave[2] = gtest
        dsave[3] = gx
        dsave[4] = gy
        dsave[5] = finit
        dsave[6] = fx
        dsave[7] = fy
        dsave[8] = stx
        dsave[9] = sty
        dsave[10] = stmin
        dsave[11] = stmax
        dsave[12] = width
        dsave[13] = width1
        return stp, task, isave, dsave

    # A modified function is used to predict the step during the
    # first stage if a lower function value has been obtained but
    # the decrease is not sufficient.
    if stage == 1 and f <= fx and f > ftest:
        # Define the modified function and derivative values.
        fm = f - stp*gtest
        fxm = fx - stx*gtest
        fym = fy - sty*gtest
        gm = g - gtest
        gxm = gx - gtest
        gym = gy - gtest

        # Call dcstep to update stx, sty, and to compute the new step.
        stx, fxm, gxm, sty, fym, gym, stp, fm, gm, brackt, stmin, stmax = dcstep(stx, fxm, gxm, sty, fym, gym, stp, fm, gm, brackt, stmin, stmax)

        # Reset the function and derivative values for f.
        fx = fxm + stx*gtest
        fy = fym + sty*gtest
        gx = gxm + gtest
        gy = gym + gtest

    else:
        # Call dcstep to update stx, sty, and to compute the new step.
        stx, fx, gx, sty, fy, gy, stp, f, g, brackt, stmin, stmax = dcstep(stx, fx, gx, sty, fy, gy, stp, f, g, brackt, stmin, stmax)

    # Decide if a bisection step is needed.
    if brackt:
        if abs(sty-stx) >= 0.66*width1: 
            stp = stx + 0.5*(sty-stx)
        width1 = width
        width = abs(sty-stx)

    # Set the minimum and maximum steps allowed for stp.
    if brackt:
        stmin = min(stx,sty)
        stmax = max(stx,sty)
    else:
        stmin = stp + xtrapl*(stp-stx)
        stmax = stp + xtrapu*(stp-stx)

    # Force the step to be within the bounds stpmax and stpmin.
    stp = max(stp, stpmin)
    stp = min(stp, stpmax)

    # If further progress is not possible, let stp be the best
    # point obtained during the search.
    if brackt and (stp <= stmin or stp >= stmax) or (brackt and stmax-stmin <= xtol*stmax): 
        stp = stx

    # Obtain another function and derivative.
    task = 'FG'

    # Save local variables.
    isave[1] = brackt
    isave[2] = stage
    dsave[1] = ginit
    dsave[2] = gtest
    dsave[3] = gx
    dsave[4] = gy
    dsave[5] = finit
    dsave[6] = fx
    dsave[7] = fy
    dsave[8] = stx
    dsave[9] = sty
    dsave[10] = stmin
    dsave[11] = stmax
    dsave[12] = width
    dsave[13] = width1
    return stp, task, isave, dsave



"""
Linesearch methods guaranteeing satisfaction of the strong Wolfe conditions.
"""

import numpy
import _pycsrch

class StrongWolfeLineSearch:
    """
    A general-purpose linesearch procedure enforcing the strong
    Wolfe conditions

    f(x+td) <= f(x) + ftol * t * <g(x),d>    (Armijo condition)

    | <g(x+td),d> | <= gtol * | <g(x),d> |   (curvature condition)

    This is a Python interface to the More and Thuente linesearch.

    Instantiate as follows

    SWLS = StrongWolfeLineSearch(f, g, d, obj, grad)

    where

    * f     is the objective value    at the current iterate x
    * g     is the objective gradient at the current iterate x
    * d     is the current search direction
    * obj   is a scalar function used to evaluate the value of
            the objective at x + t d, given t.
    * grad  is a scalar function used to evaluate the gradient
            of the objective at x + t d, given t.

    :keywords:

        :ftol:    the constant used in the Armijo condition (1e-3)
        :gtol:    the constant used in the curvature condition (0.9)
        :xtol:    a minimal relative step bracket length (1e-10)
        :stp:     an initial step value (1.0)
        :stpmin:  the initial lower bound of the bracket
        :stpmax:  the initial upper bound of the bracket

    To ensure existence of a step satisfying the strong Wolfe
    conditions, d should be a descent direction for f at x and
    ftol <= gtol.

    The final value of the step will be held in SWLS.stp

    In case an error happens, the return value SWLS.stp will be set
    to None and SWLS.message will describe what happened.

    After the search, SWLS.armijo will be set to True if the step
    computed satisfies the Armijo condition and SWLS.curvature will
    be set to True if the step satisfies the curvature condition.
    """
    def __init__(self, f, g, d, obj, grad, **kwargs):

        # Mandatory arguments
        self.f = f   # Function value f(xk)
        self.g = g   # Gradient of f at xk
        self.d = d   # Direction along which to search

        self.obj  = obj   # To evaluate function value
        self.grad = grad  # To evaluate function gradient

        # Optional arguments
        self.ftol   = kwargs.get('ftol', 1.0e-4)
        self.gtol   = kwargs.get('gtol', 0.9)
        self.xtol   = kwargs.get('xtol', 1.0e-1)
        self.stp    = kwargs.get('stp', 1.0)
        self.stpmin = kwargs.get('stpmin', 0.0)

        self.slope  = numpy.dot(self.g, self.d)
        self.stpmax = kwargs.get('stpmax',
                                  max(4*min(self.stp,1.0),
                                       0.1*self.f/(- self.slope*self.ftol)))

        # Initialize context object
        self.context = _pycsrch.Init(self.ftol,
                                      self.gtol,
                                      self.xtol,
                                      self.stp,
                                      self.stpmin,
                                      self.stpmax)

        self.armijo = False
        self.curvature = False
        self.message = None

    def search(self):

        if self.slope >= 0.0:
            self.stp = None
            self.message  = 'Direction is not a descent direction. '
            self.message += 'Slope = %-g' % self.slope
            return

        self.stp, task = self.context.csrch(self.f, self.slope)

        while task[:2] == 'FG':
            #print '  ls trying step = ', self.stp
            self.f = self.obj(self.stp)
            self.g = self.grad(self.stp)
            self.slope = numpy.dot(self.g, self.d)
            self.stp, task = self.context.csrch(self.f, self.slope)

        if task[:4] == 'CONV':     # Strong Wolfe conditions satisfied
            self.armijo = True
            self.curvature = True
        elif task[:4] == 'WARN':   # Armijo condition only is satisfied
            self.armijo = True
        self.message = task
        return



# A General module that implements various linesearch schemes
# D. Orban, Montreal, Jan. 2005.

__docformat__ = 'restructuredtext'

class LineSearch:
    """
    A generic linesearch class. Most methods of this
    class should be overridden by subclassing.
    """

    def __init__(self, **kwargs):
        self._id = 'Generic Linesearch'
        return

    def _test(self, func, x, d, slope, f = None, t = 1.0, **kwargs):
        """
        Given a descent direction d for function func at the
        current iterate x, see if the steplength t satisfies
        a specific linesearch condition.

        Must be overridden.
        """
        return True # Must override

    def search(self, func, x, d, slope, f = None, **kwargs):
        """
        Given a descent direction d for function func at the
        current iterate x, compute a steplength t such that
        func(x + t * d) satisfies a linesearch condition
        when compared to func(x). The value of the argument
        slope should be the directional derivative of func in
        the direction d: slope = f'(x;d) < 0. If given, f should
        be the value of func(x). If not given, it will be evaluated.

        func can point to a defined function or be a lambda function.
        For example, in the univariate case::

            test(lambda x: x**2, 2.0, -1, 4.0)
        """
        # Must override
        if slope >= 0.0:
            raise ValueError("Direction must be a descent direction")
            return None
        t = 1.0
        while not self._test(func, x, d, f = f, t = t, **kwargs):
            pass
        return t


class ArmijoLineSearch(LineSearch):
    """
    An Armijo linesearch with backtracking. This class implements the simple
    Armijo test

    f(x + t * d) <= f(x) + t * beta * f'(x;d)

    where 0 < beta < 1/2 and f'(x;d) is the directional derivative of f in the
    direction d. Note that f'(x;d) < 0 must be true.

    :keywords:

        :beta:      Value of beta (default 0.001)
        :tfactor:   Amount by which to reduce the steplength
                    during the backtracking (default 0.5).
    """

    def __init__(self, **kwargs):
        LineSearch.__init__(self, **kwargs)
        self.beta = max(min(kwargs.get('beta', 1.0e-4), 0.5), 1.0e-10)
        self.tfactor = max(min(kwargs.get('tfactor', 0.1), 0.999), 1.0e-3)
        return

    def _test(self, func, x, d, slope, f = None, t = 1.0, **kwargs):
        """
        Given a descent direction d for function func at the
        current iterate x, see if the steplength t satisfies
        the Armijo linesearch condition.
        """
        if f is None:
            f = func(x)
        f_plus = func(x + t * d)
        return (f_plus <= f + t * self.beta * slope)

    def search(self, func, x, d, slope, f = None, **kwargs):
        """
        Given a descent direction d for function func at the
        current iterate x, compute a steplength t such that
        func(x + t * d) satisfies the Armijo linesearch condition
        when compared to func(x). The value of the argument
        slope should be the directional derivative of func in
        the direction d: slope = f'(x;d) < 0. If given, f should
        be the value of func(x). If not given, it will be evaluated.

        func can point to a defined function or be a lambda function.
        For example, in the univariate case:

            `test(lambda x: x**2, 2.0, -1, 4.0)`
        """
        if f is None:
            f = func(x)
        if slope >= 0.0:
            raise ValueError("Direction must be a descent direction")
            return None
        t = 1.0
        while not self._test(func, x, d, slope, f = f, t = t, **kwargs):
            t *= self.tfactor
        return t




# Test
if __name__ == '__main__':

    # Simple example:
    #    steepest descent method
    #    with Armijo backtracking
    from numpy import array, dot
    from nlpy.tools.norms import norm_infty

    def rosenbrock(x):
        return 10.0 * (x[1]-x[0]**2)**2 + (1-x[0])**2

    def rosenbrockxy(x,y):
        return rosenbrock((x,y))

    def rosengrad(x):
        return array([ -40.0 * (x[1] - x[0]**2) * x[0] - 2.0 * (1-x[0]),
                        20.0 * (x[1] - x[0]**2) ], 'd')

    ALS = ArmijoLineSearch(tfactor = 0.2)
    x = array([-0.5, 1.0], 'd')
    xmin = xmax = x[0]
    ymin = ymax = x[1]
    f = rosenbrock(x)
    grad = rosengrad(x)
    d = -grad
    slope = dot(grad, d)
    t = 0.0
    tlist = []
    xlist = [x[0]]
    ylist = [x[1]]
    iter = 0
    print('%-d\t%-g\t%-g\t%-g\t%-g\t%-g\t%-g' % (iter, f, norm_infty(grad), x[0], x[1], t, slope))
    while norm_infty(grad) > 1.0e-5:

        iter += 1

        # Perform linesearch
        t = ALS.search(rosenbrock, x, d, slope, f = f)
        tlist.append(t)

        # Move ahead
        x += t * d
        xlist.append(x[0])
        ylist.append(x[1])
        xmin = min(xmin, x[0])
        xmax = max(xmax, x[0])
        ymin = min(ymin, x[1])
        ymax = max(ymax, x[1])
        f = rosenbrock(x)
        grad = rosengrad(x)
        d = -grad
        slope = dot(grad, d)
        print('%-d\t%-g\t%-g\t%-g\t%-g\t%-g\t%-g' % (iter, f, norm_infty(grad), x[0], x[1], t, slope))

    try:
        from pylab import *
    except:
        import sys
        sys.stderr.write('If you had Matplotlib, you would be looking\n')
        sys.stderr.write('at a contour plot right now...\n')
        sys.exit(0)
    xx = arange(-1.5, 1.5, 0.01)
    yy = arange(-0.5, 1.5, 0.01)
    XX, YY = meshgrid(xx, yy)
    ZZ = rosenbrockxy(XX, YY)
    plot(xlist, ylist, 'r-', lw = 1.5)
    plot([xlist[0]], [ylist[0]], 'go', [xlist[-1]], [ylist[-1]], 'go')
    contour(XX, YY, ZZ, 30, linewidths = 1.5, alpha = 0.75, origin = 'lower')
    title('Steepest descent with Armijo linesearch on the Rosenbrock function')
    show()





# There are some modifications and/or extensions from what's in the
# paper (these may or may not be extensions of the cg_descent code
# that can be downloaded from Hager's site; his code has undergone
# numerous revisions since publication of the paper):
#   cgdescent: the termination condition employs a "unit-correct"
#     expression rather than a condition on gradient
#     components---whether this is a good or bad idea will require
#     additional experience, but preliminary evidence seems to suggest
#     that it makes "reasonable" choices over a wider range of problem
#     types.
#   linesearch: the Wolfe conditions are checked only after alpha is
#     generated either by quadratic interpolation or secant
#     interpolation, not when alpha is generated by bisection or
#     expansion. This increases the likelihood that alpha will be a
#     good approximation of the minimum.
#   linesearch: In step I2, we multiply by psi2 only if the convexity
#     test failed, not if the function-value test failed. This
#     prevents one from going uphill further when you already know
#     you're already higher than the point at alpha=0.
#   both: checks for Inf/NaN function values
#   both: support maximum value of alpha (equivalently, c). This
#     facilitates using these routines for constrained minimization
#     when you can calculate the distance along the path to the
#     disallowed region. (When you can't easily calculate that
#     distance, it can still be handled by returning Inf/NaN for
#     exterior points. It's just more efficient if you know the
#     maximum, because you don't have to test values that won't
#     work.) The maximum should be specified as the largest value for
#     which a finite value will be returned.  See, e.g., limits_box
#     below.  The default value for alphamax is Inf. See alphamaxfunc
#     for cgdescent and alphamax for linesearch_hz.
from math import isnan, ceil, log
from numpy import spacing

DEFAULTDELTA = 0.1
DEFAULTSIGMA = 0.9 
eps = spacing

# Generate initial guess for step size (HZ, stage I0)
def alphainit(alpha, x, gr, f_x, psi0=0.01):
    if isnan(alpha):
        alpha = 1
        gr_max = maximum(abs(gr))
        if gr_max != 0.0:
            x_max = maximum(abs(x))
            if x_max != 0.0:
                alpha = psi0 * x_max / gr_max
            elif f_x != 0.0:
                alpha = psi0 * abs(f_x) / norm(gr)
    return alpha

# Code used to use phi, a 1-parameter function induced by f and s
# Need to pass s as an explicit parameter
def alphatry(alpha, d, x, s, xtmp, gtmp, lsr, psi1=0.2, psi2=2, psi3=0.1, iterfinitemax=ceil(-log(eps(1), 2)), alphamax=float('Inf'), display=False):

    phi0 = lsr.value[0]
    dphi0 = lsr.slope[0]

    alphatest = psi1 * alpha
    alphatest = min(alphatest, a,
            alphamax=self.stpmaxlphamax)

    # Use xtmp here
    phitest = d.f(x + alphatest * s)

    a = (phitest - alphatest * dphi0 - phi0) / alphatest**2 # quadratic fit
    if display:
        print(("quadfit: alphatest = ", alphatest,
                ", phi0 = ", phi0,
                ", phitest = ", phitest,
                ", quadcoef = ", a))

    mayterminate = False
    if a > 0 and phitest <= phi0:
        alpha = -dphi0 / 2. / a # if convex, choose minimum of quadratic
        if alpha == 0:
            error("alpha is zero. dphi0 = ", dphi0, ", a = ", a)
        if alpha <= alphamax:
            mayterminate = True
        else:
            alpha = alphamax
            mayterminate = False
        if display:
            print(("alpha guess (quadratic): ", alpha,
                    ",(mayterminate = ", mayterminate, ")"))
    else:
        if phitest > phi0:
            alpha = alphatest
        else:
            alpha *= psi2 # if not convex, expand the interval

    alpha = min(alphamax, alpha)
    if display:
        print(("alpha guess (expand): ", alpha))
    return alpha, mayterminate


class LSR:
    def __init__(self, value, slope, alpha):
        self.value = value
        self.slope = slope
        self.alpha = alpha

def hz_linesearch(df, c, mayterminate, delta=DEFAULTDELTA, sigma=DEFAULTSIGMA, alphamax=float('Inf'), rho=5., epsilon=1e-6, gamma=0.66, linesearchmax=50, psi3=0.1, iterfinitemax=ceil(-log(eps(1), 2)), display=False):

    lsr = []  # List consisting of LSR objects

    phi0, dphi0 = linefunc(df, 0, calc_grad=True)
    lsr.append(LSR(phi0, dphi0, 0))

    philim = phi0 + epsilon * abs(phi0)
    assert c > 0
    assert not isnan(c) and c <= alphamax

    phic, dphic = linefunc(df, c, calc_grad=True)
    iterfinite = 1

    lsr.append(LSR(phic, dphic, c))

    # If c was generated by quadratic interpolation, check whether it
    # satisfies the Wolfe conditions
    if mayterminate and satisfies_wolfe(c, phic, dphic, phi0, dphi0, philim, delta, sigma):
        if display:
            print(("Wolfe condition satisfied on point alpha = ", c))
        return c

    # Initial bracketing step (HZ, stages B0-B3)
    isbracketed = False
    ia = 1
    ib = 2
    assert len(lsr) == 2
    iter = 1
    cold = -1
    while not isbracketed and iter < linesearchmax:
        if display:
            print(("bracketing: ia = ", ia,
                    ", ib = ", ib,
                    ", c = ", c,
                    ", phic = ", phic,
                    ", dphic = ", dphic))
        if dphic >= 0:
            # We've reached the upward slope, so we have b; examine
            # previous values to find a
            ib = len(lsr)-1
            for i in range(ib, -1, -1):
                if lsr[i].value <= philim:
                    ia = i
                    break

            isbracketed = True
        elif lsr[-1].value > philim:
            # The value is higher, but the slope is downward, so we must
            # have crested over the peak. Use bisection.
            ib = len(lsr)-1
            ia = ib - 1
            if c != lsr[ib].alpha or lsr[ib].slope >= 0:
                raise ValueError("c = ", c, ", lsr = ", lsr)

            ia, ib = bisect(df, lsr, ia, ib, philim, display)
            isbracketed = True
        else:
            # We'll still going downhill, expand the interval and try again
            cold = c
            c *= rho
            if c > alphamax:
                if display:
                    print("bracket: exceeding alphamax, truncating")
                c = alphamax

            phic, dphic = linefunc(df, c, calc_grad=True)
            iterfinite = 1

            if dphic < 0 and c == alphamax:
                # We're on the edge of the allowed region, and the
                # value is still decreasing. This can be due to
                # roundoff error in barrier penalties, a barrier
                # coefficient being so small that being eps() away
                # from it still doesn't turn the slope upward, or
                # mistakes in the user's function.
                if iterfinite >= iterfinitemax:
                    print("Warning: failed to expand interval to bracket with finite values. If this happens frequently, check your function and gradient.")
                    print(("c = ", c,
                            ", alphamax = ", alphamax,
                            ", phic = ", phic,
                            ", dphic = ", dphic))
                return c
            lsr.append(LSR(phic, dphic, c))
        iter += 1

    while iter < linesearchmax:
        a = lsr[ia].alpha
        b = lsr[ib].alpha
        assert b > a
        if display:
            print(("linesearch: ia = ", ia,
                    ", ib = ", ib,
                    ", a = ", a,
                    ", b = ", b,
                    ", phi(a) = ", lsr[ia].value,
                    ", phi(b) = ", lsr[ib].value))

        if b - a <= eps(b):
            return a

        iswolfe, iA, iB = secant2(df, lsr, ia, ib, philim, delta, sigma, display)
        if iswolfe:
            return lsr[iA].alpha

        A = lsr[iA].alpha
        B = lsr[iB].alpha
        assert B > A
        if B - A < gamma * (b - a):
            if display:
                print("Linesearch: secant succeeded")
            ia = iA
            ib = iB
        else:
            # Secant is converging too slowly, use bisection
            if display:
                println("Linesearch: secant failed, using bisection")

            c = (A + B) / 2.
            # phic = phi(gphi, c) # TODO: Replace
            phic, dphic = linefunc(df, c, calc_grad=True)
            assert not isnan(phic)
            lsr.append(LSR(phic, dphic, c))
            ia, ib = update(df, lsr, iA, iB, len(lsr)-1, philim, display)

        iter += 1
    error("Linesearch failed to converge")

# Check Wolfe & approximate Wolfe
def satisfies_wolfe(c, phic, dphic, phi0, dphi0, philim, delta=DEFAULTDELTA, sigma=DEFAULTSIGMA):
    wolfe1 = delta * dphi0 >= (phic - phi0) / c and dphic >= sigma * dphi0
    wolfe2 = (2.0 * delta - 1.0) * dphi0 >= dphic >= sigma * dphi0 and phic <= philim
    return wolfe1 or wolfe2

# HZ, stages S1-S4
def secant(a, b, dphia, dphib):
    return (a * dphib - b * dphia) / (dphib - dphia)

def secant_(lsr, ia, ib):
    return secant(lsr[ia].alpha, lsr[ib].alpha, lsr[ia].slope, lsr[ib].slope)

# phi
def secant2(df, lsr, ia, ib, philim, delta=DEFAULTDELTA, sigma=DEFAULTSIGMA, display=False):

    phi0 = lsr[0].value
    dphi0 = lsr[0].slope
    a = lsr[ia].alpha
    b = lsr[ib].alpha
    dphia = lsr[ia].slope
    dphib = lsr[ib].slope
    assert dphia < 0
    assert dphib >= 0
    c = secant(a, b, dphia, dphib)
    if display:
        print(("secant2: a = ", a, ", b = ", b, ", c = ", c))

    assert not isnan(c)
    # phic = phi(tmpc, c) # Replace

    phic, dphic = linefunc(df, c, calc_grad=True)
    assert not isnan(phic)
    lsr.append(LSR(phic, dphic, c))
    ic = len(lsr)-1
    if satisfies_wolfe(c, phic, dphic, phi0, dphi0, philim, delta, sigma):
        if display:
            print("secant2: first c satisfied Wolfe conditions")
        return True, ic, ic

    iA, iB = update(df, lsr, ia, ib, ic, philim, display)
    if display:
        print(("secant2: iA = ", iA, ", iB = ", iB, ", ic = ", ic))

    a = lsr[iA].alpha
    b = lsr[iB].alpha
    doupdate = false
    if iB == ic:
        # we updated b, make sure we also update a
        c = secant_(lsr, ib, iB)
    elif iA == ic:
        # we updated a, do it for b too
        c = secant_(lsr, ia, iA)
    if a <= c <= b:
        if display:
            print(("secant2: second c = ", c))

        phic, dphic = linefunc(df, c, calc_grad=True)
        assert not isnan(phic)
        lsr.append(LSR(phic, dphic, c))
        ic = len(lsr)-1
        # Check arguments here
        if satisfies_wolfe(c, phic, dphic, phi0, dphi0, philim, delta, sigma):
            if display:
                print("secant2: second c satisfied Wolfe conditions")
            return True, ic, ic

        iA, iB = update(df, lsr, iA, iB, ic, philim, display)

    if display: 
        print(("secant2 output: a = ", lsr[iA].alpha, ", b = ", lsr[iB].alpha))

    return False, iA, iB

# HZ, stages U0-U3
# Given a third point, pick the best two that retain the bracket
# around the minimum (as defined by HZ, eq. 29)
# b will be the upper bound, and a the lower bound
def update(df, lsr, ia, ib, ic, philim, display=False):

    a = lsr[ia].alpha
    b = lsr[ib].alpha
    # Debugging (HZ, eq. 4.4):
    assert lsr[ia].slope < 0
    assert lsr[ia].value <= philim
    assert lsr[ib].slope >= 0
    assert b > a
    c = lsr[ic].alpha
    phic = lsr[ic].value
    dphic = lsr[ic].slope
    if display:
        print(("update: ia = ", ia,
                ", a = ", a,
                ", ib = ", ib,
                ", b = ", b,
                ", c = ", c,
                ", phic = ", phic,
                ", dphic = ", dphic))

    if c < a or c > b:
        return ia, ib, 0, 0  # it's out of the bracketing interval

    if dphic >= 0:
        return ia, ic, 0, 0  # replace b with a closer point

    # We know dphic < 0. However, phi may not be monotonic between a
    # and c, so check that the value is also smaller than phi0.  (It's
    # more dangerous to replace a than b, since we're leaving the
    # secure environment of alpha=0; that's why we didn't check this
    # above.)
    if phic <= philim:
        return ic, ib, 0, 0  # replace a

    # phic is bigger than phi0, which implies that the minimum
    # lies between a and c. Find it via bisection.
    return bisect(df, lsr, ia, ic, philim, display)

# HZ, stage U3 (with theta=0.5)
def bisect(df, lsr, ia, ib, philim, display=False):

    gphi = nan(T)
    a = lsr[ia].alpha
    b = lsr[ib].alpha
    # Debugging (HZ, conditions shown following U3)
    assert lsr[ia].slope < 0
    assert lsr[ia].value <= philim
    assert lsr[ib].slope < 0       # otherwise we wouldn't be here
    assert lsr[ib].value > philim
    assert b > a

    while b - a > eps(b):
        if display:
            print(("bisect: a = ", a, ", b = ", b, ", b - a = ", b - a))

        d = (a + b) / 2.
        phid, gphi = linefunc(df, d, calc_grad=True)
        assert isfinite(phid)
        lsr.append(LSR(phid, gphi, d))
        id = len(lsr)-1
        if gphi >= 0:
            return ia, id # replace b, return

        if phid <= philim:
            a = d # replace a, but keep bisecting until dphib > 0
            ia = id
        else:
            b = d
            ib = id

    return ia, ib

# Define one-parameter function for line searches
def linefunc(df, alpha, calc_grad):

    if calc_grad:
        return df.phi_dphi(alpha)
    else:
        return df.phi(alpha)



from .optimisation_algorithm import *
from math import sqrt

class LinearOperator(object):

    def __init__(self, matvec):
        self.matvec = matvec

    def __mul__(self,x):
        return self.matvec(x)

    def __rmul__(self,x):
        return NotImplemented

    def __call__(self,x):
        return self.matvec(x)


def dual_to_primal(x):
    return x.primal()


class LimitedMemoryInverseHessian(LinearOperator):
    '''
    This class implements the limit-memory BFGS approximation of the inverse Hessian.
    '''
    def __init__(self, Hinit, mem_lim = 10, theta = 1.0, theta_rule = 1):
        self.Hinit = Hinit
        self.mem_lim = mem_lim
        self.y   = []
        self.s   = []
        self.rho = []
        self.theta = theta
        self.theta_rule = theta_rule

    def __len__(self):
        assert( len(self.y) == len(self.s) )
        return len(self.y)

    def __getitem__(self,k):
        if k==0:
            return self.Hinit
        return (self.rho[k-1], self.y[k-1], self.s[k-1])

    def update(self,yk, sk):
        if self.mem_lim == 0:
            return
        if len(self) == self.mem_lim:
            self.y   = self.y[1:]
            self.s   = self.s[1:]
            self.rho = self.rho[1:]
        self.y.append(yk)
        self.s.append(sk)
        self.rho.append( 1./ yk.apply(sk) )
        self.theta = self.compute_theta()

    def compute_theta(self):
        sty, yk, sk = 1./self.rho[-1],self.y[-1], self.s[-1]
        if self.theta_rule != 0:
            t1 = sty / yk.apply(yk.primal())
            if self.theta_rule == 1:
                return t1
        t0 = sk.norm()**2 / sty
        if self.theta_rule == 0: return t0
        if self.theta_rule == 2: return (t0 + t1) / 2
        if self.theta_rule == 3: return sqrt(t0 * t1)


    def matvec(self,x,k = -1):
        if k == -1:
            k = len(self)
        if k == 0:
            return self.theta * (self.Hinit * x)
        rhok, yk, sk = self[k]
        t = x - rhok * x.apply(sk) * yk
        t = self.matvec(t, k-1)
        t = t - rhok * yk.apply(t) * sk
        t = t + rhok * x.apply(sk) * sk
        return t

class BFGS(OptimisationAlgorithm):
    """
        Implements the BFGS method.
     """
    def __init__(self, problem, initial_point = None, options={}, hooks={}, **args):
        '''
        Initialises the L-BFGS algorithm.

        Valid options are:

         * Hinit: Initial approximation of the inverse Hessian.
         * options: A dictionary containing additional options for the steepest descent algorithm. Valid options are:
            - jtol: Functional reduction stopping tolerance: |j - j_prev| < tol. Default: 1e-4.
            - rjtol: Functional reduction stopping tolerance: |j - j_prev| < tol. Default: 1e-4.
            - gtol: Gradient norm stopping tolerance: ||grad j|| < gtol.
            - rgtol: Relative version of the gradient stopping criterion.
            - maxiter: Maximum number of iterations before the algorithm terminates. Default: 200.
            - disp: dis/enable outputs to screen during the optimisation. Default: True
            - line_search: defines the line search algorithm to use. Default: strong_wolfe
            - line_search_options: additional options for the line search algorithm. The specific options read the help
              for the line search algorithm.
            - an optional callback method which is called after every optimisation iteration.
         * hooks: A dictionariy containing user-defined "hook" functions that are called at certain events during the optimisation.
            - before_iteration: Is called after before each iteration.
            - after_iteration: Is called after each each iteration.
          '''
        # Set the default options values
        self.problem = problem
        self.set_options(options)
        if self.options["Hinit"] == "default":
            Hinit = LinearOperator(dual_to_primal)
        elif self.options["Hinit"] == "l2":
            print("Warning: Using l2 as Hinit is experimental and will be slow")
            Hinit = LinearOperator( lambda v: v.primal().__class__(v.data) )  # FIXME: Find a better way how to get the PrimalVector class
        else:
            Hinit = LinearOperator(self.options["Hinit"])

        self.linesearch = get_line_search_method(self.options['line_search'], self.options['line_search_options'])
        self.data = {'control'   : initial_point,
                     'iteration' : 0,
                     'lbfgs'     : LimitedMemoryInverseHessian(Hinit, self.options['mem_lim']) }



    @classmethod
    def default_options(cls):
        # this is defined as a function to prevent defaults from being changed at runtime.
        default = OptimisationAlgorithm.default_options()
        default.update(
            # generic parameters:
            {"jtol"                   : 1e-4,
             "rjtol"                  : 1e-6,
             "gtol"                   : 1e-4,
             "rgtol"                  : 1e-5,
             "maxiter"                :  200,
             "display"                :    2,
             "line_search"            : "strong_wolfe",
             "line_search_options"    : {"ftol": 1e-3, "gtol": 0.9, "xtol": 1e-1, "start_stp": 1},
             "callback"               : None,
             "record"                 : ("grad_norm", "objective"),
             # method specific parameters:
             "mem_lim"                : 5,
             "Hinit"                  : "default",
             })
        return default

    def __str__(self):
        s = "L-BFGS method.\n"
        s += "-"*30 + "\n"
        s += "Line search:\t\t %s\n" % self.options['line_search']
        s += "Maximum iterations:\t %i\n" % self.options['maxiter']
        return s

    def solve(self):
        '''
            Arguments:
             * problem: The optimisation problem.

            Return value:
              * solution: The solution to the optimisation problem
         '''
        self.display( self.__str__(), 1)

        objective = self.problem.obj
        options = self.options

        Hk = self.data['lbfgs']
        xk = self.data['control']
        it = self.data['iteration']

        # compute initial objective and gradient
        J = objective(xk)
        dJ_xk = objective.derivative(xk)
        dJ_norm = dJ_xk.primal_norm()
        self.update({'objective' : J,
                     'initial_J' : J,
                     'grad_norm' : dJ_norm,
                     'initial_grad_norm': dJ_norm})
        self.record_progress()

        # Start the optimisation loop
        while self.check_convergence() == 0:
            self.display(self.iter_status, 2)
            # compute search direction
            pk = - (Hk * dJ_xk)
            if it == 0:
                # then normalize;
                pk.scale( 1./  dJ_xk.primal_norm())

            # do a line search and update
            xk, ak = self.do_linesearch(objective, xk, pk)
            pk.scale(ak)
            J, oldJ = objective(xk), J

            # evaluate gradient at the new point
            dJ_xk, dJ_old = objective.derivative(xk), dJ_xk
            yk = dJ_xk - dJ_old

            # update the approximate Hessian
            Hk.update(yk, pk)

            it += 1

            # store current iteration variables
            self.update({'iteration' : it,
                         'control'   : xk,
                         'grad_norm' : dJ_xk.primal_norm(),
                         'delta_J'   : oldJ-J,
                         'objective' : J,
                         'lbfgs'     : Hk })
            self.record_progress()
        self.display(self.convergence_status, 1)
        self.display(self.iter_status, 1)
        return self.data




from .line_search import LineSearch

class FixedLineSearch(LineSearch):
    def __init__(self, start_stp = 1.0):
        '''
        This class implements a dummy line search algorithm whose steps 
        are simply the initial guess.
        '''

        self.start_stp    = start_stp

    def search(self, phi, phi_dphi, phi_dphi0):
        ''' Performs the dummy line search on the function phi. 

            The return value is the starting step. 
        '''
        return self.start_stp   



#!/usr/bin/env python

def configuration(parent_package='',top_path=None):
    import numpy
    from numpy.distutils.misc_util import Configuration
    from numpy.distutils.system_info import get_info, NotFoundError
    import os

    config = Configuration('ls', parent_package, top_path)

    # Get info from site.cfg
    blas_info = get_info('blas_opt',0)
    if not blas_info:
        print('No blas info found')

    libcsrch_src = ['dcsrch.f', 'dcstep.f']
    pycsrch_src = ['_pycsrch.c']

    libmcsrch_src = ['mcsrch.f', 'mcstep.f']
    pymcsrch_src = ['_pymcsrch.c']

    config.add_library(
        name='nlpy_csrch',
        sources=[os.path.join('src',name) for name in libcsrch_src],
        libraries=[],
        library_dirs=[],
        include_dirs=['src'],
        extra_info=blas_info,
        )

    config.add_extension(
        name='_pycsrch',
        sources=[os.path.join('src',name) for name in pycsrch_src],
        depends=[],
        libraries=['nlpy_csrch'],
        library_dirs=[],
        include_dirs=['src'],
        extra_info=blas_info,
        )

    config.add_library(
        name='nlpy_mcsrch',
        sources=[os.path.join('src',name) for name in libmcsrch_src],
        libraries=[],
        library_dirs=[],
        include_dirs=['src'],
        extra_info=blas_info,
        )

    config.add_extension(
        name='_pymcsrch',
        sources=[os.path.join('src',name) for name in pymcsrch_src],
        depends=[],
        libraries=['nlpy_mcsrch'],
        library_dirs=[],
        include_dirs=['src'],
        extra_info=blas_info,
        )

    config.make_config_py()
    return config

if __name__ == '__main__':
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



class Solution(dict):
    ''' Stores the Solution of an optimisation run. '''

    def __str__(self):
        s = ""
        for k, v in list(self.items()):
            s += "%s:\t\t%s\n" % (k, v)
        return s




class LineSearch:
    ''' A base class for a line search implementation. '''

    def search(self, phi, phi_dphi):
        ''' This method performs the line search. 

            The two passed-in arguments are functions:
               phi:  [0, oo] -> R
               phi_dphi: [0, oo] -> (R, R)
            where phi_dphi evaluates both phi and its derivative.

            For multidimensional optimisation problems min_m f(m), dim(m) >= 1, phi is typically defined as:
            phi(a) = f(m_ + a*s),
            where m_ and s are the current parameter point and search direction, respectively and a \in R.

            The return value must be the computed step size.
        '''
        raise NotImplementedError("Must be overloaded.")




from moola.linalg import Vector
from moola.misc import events
from math import sqrt
from ufl.form import Form
import dolfin

class IdentityMap(object):

    def primal_map(self, x, b):
        x.zero()
        x.axpy(1, b)

    def dual_map(self, x):
        return x.copy()

class RieszMap(object):

    def __init__(self, V, inner_product="L2", map_operator=None, inverse = "default"):
        self.V = V

        if inner_product is not "custom":
            u = dolfin.TrialFunction(V)
            v = dolfin.TestFunction(V)

            if isinstance(V, dolfin.cpp.function.MultiMeshFunctionSpace):
                default_forms = {"L2":   dolfin.inner(u, v)*dolfin.dX,
                                 "H0_1": dolfin.inner(dolfin.grad(u), dolfin.grad(v))*dolfin.dX,
                                 "H1":  (dolfin.inner(u, v) + dolfin.inner(dolfin.grad(u),
                                                                           dolfin.grad(v)))*dolfin.dX,
                }
            else:
                default_forms = {"L2":   dolfin.inner(u, v)*dolfin.dx,
                                 "H0_1": dolfin.inner(dolfin.grad(u), dolfin.grad(v))*dolfin.dx,
                                 "H1":  (dolfin.inner(u, v) + dolfin.inner(dolfin.grad(u),
                                                                           dolfin.grad(v)))*dolfin.dx,
                }

            form = default_forms[inner_product]
            if hasattr(form.arguments()[0], "_V_multi"):
                map_operator = dolfin.assemble_multimesh(form)
            else:
                map_operator = dolfin.assemble(form)
        self.map_operator = map_operator
        if inverse in ("default", "lu"):
            self.map_solver = dolfin.LUSolver(self.map_operator)

        elif inverse == "jacobi":
            self.map_solver = dolfin.PETScKrylovSolver()
            self.map_solver.set_operator(self.map_operator)
            self.map_solver.ksp().setType("preonly")
            self.map_solver.ksp().getPC().setType("jacobi")

        elif inverse == "sor":
            self.map_solver = dolfin.PETScKrylovSolver()
            self.map_solver.set_operator(self.map_operator)
            self.map_solver.ksp().setType("preonly")
            self.map_solver.ksp().getPC().setType("sor")

        elif inverse == "amg":
            self.map_solver = dolfin.PETScKrylovSolver()
            self.map_solver.set_operator(self.map_operator)
            self.map_solver.ksp().setType("preonly")
            self.map_solver.ksp().getPC().setType("hypre")
            
        elif isinstance(inverse, dolfin.GenericMatrix):
            self.map_solver = dolfin.PETScKrylovSolver()
            self.map_solver.set_operators(self.map_operator, inverse)
            self.map_solver.ksp().setType("preonly")
            self.map_solver.ksp().getPC().setType("mat")
            
            
        else:
            self.map_solver = inverse
        self.solver_type = inverse
    def primal_map(self, x, b):
        self.map_solver.solve(x, b)

    def dual_map(self, x):
        return self.map_operator * x


class DolfinVector(Vector):
    ''' An implementation for vectors based on Dolfin data types. '''

    def __init__(self, data, inner_product="L2", riesz_map=None):
        ''' Wraps the Dolfin object `data` in a moola.DolfinVector object. '''

        if riesz_map is None:
            if inner_product=="l2":
                riesz_map = IdentityMap()
            else:
                fn_space = data.function_space()
                riesz_map = RieszMap(fn_space, inner_product)

        self.riesz_map = riesz_map
        self.data = data
        self.version = 0

    def bump_version(self):
        self.version += 1

    def __getitem__(self, index):
        ''' Returns the value of the (local) index. '''
        return self.data.vector()[index]

    def __setitem__(self, index, value):
        ''' Sets the value of the (local) index. '''
        self.data.vector()[index] = value

    def array(self):
        ''' Returns the vector as a numpy.array object. If local=False, the
        global array must be returned in a distributed environment. '''
        return self.data.vector().array()

    def scale(self, s):
        v = self.data.vector()
        v *= s
        self.bump_version()

    def __hash__(self):
        ''' Returns a hash of the vector '''
        return hash(self.data)*100 + self.version

    def axpy(self, a, x):
        ''' Adds a*x to the function. '''
        assert self.__class__ == x.__class__

        v = self.data.vector()
        v.axpy(a, x.data.vector())
        self.bump_version()

    def zero(self):
        ''' Zeros the function. '''
        v = self.data.vector()
        v.zero()
        self.bump_version()
        return self

    def local_size(self):
        ''' Returns the (local) size of the vector. '''
        return self.data.vector().local_size()

    def size(self):
        ''' Returns the (gobal) size of the vector. '''
        return self.data.vector().size()

    def copy(self):
        return self.__class__(self.data.copy(deepcopy=True),
                              riesz_map=self.riesz_map)


class DolfinPrimalVector(DolfinVector):
    """ A class for representing primal vectors. """


    def dual(self):
        """ Returns the dual representation. """

        events.increment("Primal -> dual map")
        if isinstance(self.data, dolfin.Function):
            V = self.data.function_space()

            dual_vec = self.riesz_map.dual_map(self.data.vector())
            dual = dolfin.Function(V, dual_vec)

            return DolfinDualVector(dual, riesz_map=self.riesz_map)
        else:
            return self

    def inner(self, vec):
        """ Computes the inner product with vec. """
        assert isinstance(vec, DolfinPrimalVector)
        events.increment("Inner product")

        v = self.riesz_map.dual_map(self.data.vector())
        return v.inner(self.data.vector())

    def norm(self):
        """ Computes the vector norm induced by the inner product. """

        return sqrt(self.inner(self))
    primal_norm = norm


class DolfinDualVector(DolfinVector):
    """ A class for representing dual vectors. """

    def apply(self, primal):
        """ Applies the dual vector to a primal vector. """
        assert isinstance(primal, DolfinPrimalVector)
        return self.data.vector().inner(primal.data.vector())

    def primal(self):
        """ Returns the primal representation. """
        events.increment("Dual -> primal map")

        if isinstance(self.data, dolfin.Function):
            V = self.data.function_space()

            dual = dolfin.Function(V)
            self.riesz_map.primal_map(dual.vector(), self.data.vector())

            return DolfinPrimalVector(dual, riesz_map=self.riesz_map)
        elif isinstance(self.data, dolfin.MultiMeshFunction):
            V = self.data.function_space()

            dual = dolfin.MultiMeshFunction(V)
            self.riesz_map.primal_map(dual.vector(), self.data.vector())

            return DolfinPrimalVector(dual, riesz_map=self.riesz_map)
        else:
            return self

    def primal_norm(self):
        """ Computes the norm of the primal representation. """

        return sqrt(self.apply(self.primal()))

DolfinLinearFunctional = DolfinDualVector



class EventCounter(dict):
    ''' A simple class for counting events. '''

    def increment(self, event_name):
        try: 
            self[event_name] += 1
        except KeyError:
            self[event_name] = 1

    def __str__(self):
        print("")
        print("="*20 + "Moola Events" + "="*20)
        print("")
        s = []

        row_format ="{:>25}" * 2
        for k, v in self.items():
            s.append(row_format.format(k, v))

        return "\n".join(s)


events = EventCounter()



from .line_search import LineSearch
from .hz_linesearch import hz_linesearch

class HagerZhangLineSearch(LineSearch):
    def __init__(self, ftol=1e-4, gtol=0.9, xtol=1e-1, start_stp=1.0, stpmax=float('Inf'), display=False):
        '''
        This class implements a line search algorithm whose steps 
        satisfy the approximate Wolfe conditions
        '''

        self.ftol            = ftol 
        self.gtol            = gtol
        self.xtol            = xtol
        self.start_stp       = start_stp
        self.stpmax          = stpmax
        self.display         = display

    def search(self, phi, phi_dphi, phi_dphi0):
        ''' Performs the line search on the function phi. 

            phi must be a function [0, oo] -> R.
            phi_dphi must evaluate phi and its derivative, and 
            must be a function [0, oo] -> (R, R).

            The return value is a step that satisfies the strong Wolfe condition. 
        '''
            

        class DF(object):
            pass

        df = DF()
        df.phi = phi
        df.phi_dphi = phi_dphi

        mayterminate=True

        stp = hz_linesearch(df, c=self.start_stp, 
                mayterminate=mayterminate, 
                display=self.display,
                alphamax=self.stpmax)
        return stp



from .optimisation_algorithm import *
from math import sqrt
from IPython import embed
class TrustRegionNewtonCG(OptimisationAlgorithm):
    ''' 
    An implementation of the trust region NewtonCG method 
    described in Wright 2006, section 7. 
    '''
    def __init__(self, problem, initial_point = None, options={}):
        '''
        Initialises the trust region Newton CG method. The valid options are:
         * options: A dictionary containing additional options for the steepest descent algorithm. Valid options are:
            - maxiter: Maximum number of iterations before the algorithm terminates. Default: 200. 
            - disp: dis/enable outputs to screen during the optimisation. Default: True
            - gtol: Gradient norm stopping tolerance: ||grad j|| < gtol.
          '''

        # Set the default options values
        self.problem = problem
        self.set_options(options)

        self.data = {'control'   : initial_point,
                     'iteration' : 0}

        # validate method specific options
        assert 0 <= self.options['eta'] < 1./4
        assert self.options['tr_Dmax'] > 0
        assert 0 < self.options['tr_D0'] < self.options['tr_Dmax']
    
    def __str__(self):
        s = "Trust region Newton CG method.\n"
        s += "-"*30 + "\n"
        s += "Maximum iterations:\t %i\n" % self.options['maxiter']
        return s

    # set default parameters
    
    @classmethod
    def default_options(cls):
        # this is defined as a function to prevent defaults from being changed at runtime.
        default = OptimisationAlgorithm.default_options()
        default.update(
            # generic parameters:
            {"gtol"                   : 1e-4,
             "maxiter"                :  200,
             "display"                :    2,
             "callback"               : None,
             "record"                 : ("grad_norm"), 

             # method specific parameters:
             "tr_Dmax"                :  1e5, # overall bound on the step lengths
             "tr_D0"                  :    1, # current bound on the step length
             "eta"                    : 1./8,
             })
        return default

    def get_tau(self, obj, z, d, D, verify=True):
        """ Function to find tau such that p = pj + tau.dj, and ||p|| = D. """

        
        dot_z_d = z.inner(d)
        len_d_sqrd = d.inner(d)
        len_z_sqrd = z.inner(z)

        t = sqrt(dot_z_d**2 - len_d_sqrd * (len_z_sqrd - D**2))

        taup = (- dot_z_d + t) / len_d_sqrd
        taum = (- dot_z_d - t) / len_d_sqrd

        if verify:
            eps = 1e-8
            if abs((z+taup*d).norm()-D)/D > eps or abs((z+taum*d).norm()-D)/D > eps:
                raise ArithmeticError("Tau could not be computed accurately due to numerical errors.")

        return taup, taum

    def compute_pk_cg_steihaug(self, obj, x, D):
        ''' Solves min_pk fk + grad fk(p) + 0.5*p^T H p using the CG Steighaug method '''  
        z = x.copy()
        z.zero()
        r = obj.gradient(x)
        d = -r
        rtr = r.inner(r)
        rnorm = rtr**0.5
        eps = min(0.5, sqrt(rnorm))*rnorm  # Stopping criteria for CG

        if rnorm < eps:
            print("CG solver converged")
            return z

        cg_iter = 0
        while True:
            print("CG iteration %s" % cg_iter)
            cg_iter += 1

            Hd = obj.hessian(x)(d)
            curv = Hd.apply(d)

            # Curvatur test 
            if curv <= 0:
                print("curv <= 0.0")
                taup, taum = self.get_tau(obj, z, d, D)
                pp = z + taup*d
                pm = z + taum*d

                if self.mk(obj, xk, pp) <= self.mk(obj, xk, pm):
                    return pp, True
                else:
                    return pm, True

            alpha = rtr / curv
            z_old = z.copy()
            z.axpy(alpha, d)
            znorm = z.norm()
            print("|z_%i| = %f" % (cg_iter, znorm))

            # Trust region boundary test
            if znorm >= D:
                print("|z| >= Delta")
                tau = self.get_tau(obj, z_old, d, D)[0]
                assert tau >= 0
                return z_old + tau*d, True

            r.axpy(alpha, Hd.primal())
            rtr, rtr_old = r.inner(r), rtr
            rnorm = rtr**0.5

            # CG convergence test
            if rnorm < eps:
                print("CG solver converged")
                return z, False

            beta = rtr / rtr_old
            d = -r + beta*d


    def solve(self):
        ''' Solves the optimisation problem with the trust region Newton-CG method. 
         '''
        print(self)
            
        print("Doing another iteration")
        obj = self.problem.obj
        i = 0
        Dk = self.options['tr_D0']
        xk = self.data['control']

        while True:
            res = obj.derivative(xk)
            self.update({'grad_norm' : res.primal_norm()})
            if self.check_convergence() != 0:
                break

            self.display(self.iter_status, 2)

            # Compute trust region point
            pk, is_cauchy_point = self.compute_pk_cg_steihaug(obj, xk, Dk)

            # Evaluate trust region performance
            objxk = obj(xk)
            mkxkpk = objxk + obj.derivative(xk).apply(pk) + 0.5 * obj.hessian(xk)(pk).apply(pk)
            objxkpk = obj(xk + pk)

            rhok = (objxk - objxkpk) / (objxk - mkxkpk)

            if rhok < 1./4:
                Dk *= 1./4
                print("Decreasing trust region radius to %f." % Dk)

            elif rhok > 3./4 and is_cauchy_point:
                Dk = min(2*Dk, self.options['tr_Dmax'])
                print("Increasing trust region radius to %f." % Dk)

            if rhok > self.options['eta']:
                xk.axpy(1., pk)
                print("Trust region step accepted.")
            else:
                print("Rejecting step. Reason: trust region step did not reduce objective.")

            i += 1
            

            res = obj.derivative(xk)
            # store current iteration variables
            self.update({'iteration' : i,
                         'control'   : xk,
                         'grad_norm' : res.primal_norm()
                        })
        self.display(self.convergence_status, 1)
        self.display(self.iter_status, 1)
        return self.data



from .optimisation_algorithm import *
from numpy import sqrt

class NonLinearCG(OptimisationAlgorithm):
    ''' 
    An implementation of the Fletcher-Reeves method 
    described in Wright 2006, section 5.2. 
    '''
    def __init__(self, problem, initial_point = None, options={}, **args):
        '''
        Initialises the Fletcher-Reeves mehtod. The valid options are:
         * tol: Not supported - must be None. 
         * options: A dictionary containing additional options for the steepest descent algorithm. Valid options are:
            - maxiter: Maximum number of iterations before the algorithm terminates. Default: 200. 
            - disp: dis/enable outputs to screen during the optimisation. Default: True
            - gtol: Gradient norm stopping tolerance: ||grad j|| < gtol.
            - line_search: defines the line search algorithm to use. Default: strong_wolfe
            - line_search_options: additional options for the line search algorithm. The specific options read the help 
              for the line search algorithm.
            - an optional callback method which is called after every optimisation iteration.
          '''

        # Set the default options values
        self.problem = problem
        self.set_options(options)
        self.linesearch = get_line_search_method(self.options['line_search'], self.options['line_search_options'])
        self.data = {'control'      : initial_point,
                     'iteration'    : 0}
        self.compute_beta = _beta_rules[self.options['beta_rule']]
    
    @classmethod
    def default_options(cls):
        # this is defined as a function to prevent defaults from being changed at runtime.
        default = OptimisationAlgorithm.default_options()
        default.update(
            # generic parameters:
            {"jtol"                   : None,
             "gtol"                   : 1e-4,
             "maxiter"                :  200,
             "display"                :    2,
             "line_search"            : "strong_wolfe",
             "line_search_options"    : {},
             "callback"               : None,
             "record"                 : ("grad_norm", "objective"),

             # method specific parameters:
             "beta_rule"              : "polak-ribiere-polyak"
             })
        return default

    @classmethod
    def list_beta_rules(cls):
        return list(_beta_rules.keys())
    

    def __str__(self):
        s = "Nonlinear CG ({}).\n".format(self.options['beta_rule'])
        s += "-"*30 + "\n"
        s += "Line search:\t\t %s\n" % self.options['line_search']
        s += "Maximum iterations:\t %i\n" % self.options['maxiter']
        return s
    
    def solve(self):
        ''' Solves the optimisation problem with a (conjugate) gradient method. 
            Arguments:
             * problem: The optimisation problem.

            Return value:
              * solution: The solution to the optimisation problem 
         '''
        self.display(self.__str__(), 1)

        options = self.options
        obj = self.problem.obj

        x = self.data['control']

        J = obj(x)
        derivative = obj.derivative(x)
        gradient = derivative.primal()
        grad_norm2 = derivative.apply(gradient)

        self.update({'grad_norm' : sqrt(grad_norm2),
                     'objective' : obj(x)})
        self.record_progress()
        
        p = gradient.copy() # initial search direction
        p.scale(-1)

        # Start the optimisation loop
        it = 0
        while self.check_convergence() == 0:
            self.display(self.iter_status, 2)

            # Perform the line search
            try:
                x, alpha = self.do_linesearch(obj, x, p)
            except RuntimeError:
                # TODO: handle errors better.
                # For assuming error is due to search diretion not being a descent direction.
                self.display('Search direction is not a descent direction; restarting.', 2)
                p = -1 * gradient
                x, alpha = self.do_linesearch(obj, x, p)
                

            # gradient evaluations at new point
            derivative, old_derivative = obj.derivative(x), derivative 
            gradient, old_gradient = derivative.primal(), gradient
            grad_norm2, old_grad_norm2 = derivative.apply(gradient), grad_norm2
            
            # Compute the relaxation value
            beta = self.compute_beta(gradient, derivative, grad_norm2, p, 
                                     old_gradient, old_derivative, old_grad_norm2)

            # get new search direction:  p_new = beta * p_old - gradient
            p.scale(beta)
            p.axpy(-1., gradient)
            it += 1
            J, old_J = obj(x), J
            #update
            self.update(iteration = it, control = x, grad_norm = sqrt(grad_norm2),
                        objective = J, delta_J = old_J - J)
            self.record_progress()

        # Print the reason for convergence
        self.display(self.convergence_status, 1)
        self.display(self.iter_status, 1)
        
        return self.data



    
def _steepest_descent(gradient, derivative, grad_norm2, p,
                      old_gradient, old_derivative, old_grad_norm2):
    return 0.

def _fletcher_reeves(gradient, derivative, grad_norm2, p,
                     old_gradient, old_derivative, old_grad_norm2):
    '''
    Compute the Flether-Reeves update parameter.
    '''
    beta = grad_norm2 / old_grad_norm2
    return beta

def _hestenes_stiefel(gradient, derivative, grad_norm2, p,
                      old_gradient, old_derivative, old_grad_norm2):
    '''
    Computes the 'standard cg' beta 
    '''
    y = derivative - old_derivative
    beta = y.apply(gradient) / y.apply(p)
    return beta

def  _conjugate_descent(gradient, derivative, grad_norm2, p,
                        old_gradient, old_derivative, old_grad_norm2):
    beta = grad_norm2 / - old_derivative.apply(p)
    return beta

def  _polak_ribiere_polyak(gradient, derivative, grad_norm2, p,
                    old_gradient, old_derivative, old_grad_norm2):
    y = derivative - old_derivative
    beta = y.apply(gradient) / old_grad_norm2
    return beta

def _hager_zhang(gradient, derivative, grad_norm2, p,
                 old_gradient, old_derivative, old_grad_norm2):
    y = derivative - old_derivative
    z = y.primal()
    t1 = y.apply(p)
    t2 = 2 * y.apply(z) / t1
    beta = derivative.apply(z - t2*p) / t1
    return beta

def _dai_yuan(gradient, derivative, grad_norm2, p,
              old_gradient, old_derivative, old_grad_norm2):
    y = derivative - old_derivative
    beta = grad_norm2 / y.apply(p)
    return beta

def _liu_storey(gradient, derivative, grad_norm2, p,
                      old_gradient, old_derivative, old_grad_norm2):
    y = derivative - old_derivative
    return - y.apply(gradient) /  old_derivative.apply(p)
    

_beta_rules = {'steepest_descent'    : _steepest_descent,
               'fletcher-reeves'     : _fletcher_reeves,
               'hestenes-stiefel'    : _hestenes_stiefel,
               'conjugate_descent'   : _conjugate_descent,
               'polak-ribiere-polyak': _polak_ribiere_polyak,
               'hager-zhang'         : _hager_zhang,
               'dai-yuan'            : _dai_yuan,
               'liu-storey'          : _liu_storey}




class Vector(object):
    ''' An abstract implementation for vectors. '''

    ############# To be implement by overloaded class #####################
    def __getitem__(self, index):
        ''' Returns the value of the (local) index. '''
        raise NotImplementedError("Vector.__getitem__ is not implemented")

    def __setitem__(self, index, value):
        ''' Sets the value of the (local) index. '''
        raise NotImplementedError("Vector.__setitem__ is not implemented")

    def array(self, local=True):
        ''' Returns the vector as a numpy.array object. If local=False, the 
        global array must be returned in a distributed environment. '''
        raise NotImplementedError("Vector.array is not implemented")

    def set(self, array, local=True):
        ''' Sets the values of the vector to the values in the numpy.array. 
        If local=False, the global array must be returned in a distributed environment. '''
        raise NotImplementedError("Vector.set is not implemented")

    def scale(self, s):
        ''' Scales the vector by s. '''
        raise NotImplementedError("Vector.scale is not implemented")

    def axpy(self, a, x):
        ''' Adds a*x to the vector. '''
        raise NotImplementedError("Vector.axpy is not implemented")

    def local_size(self):
        ''' Returns the (local) size of the vector. '''
        raise NotImplementedError("Vector.__local_size__ is not implemented")

    def size(self):
        ''' Returns the (global) size of the vector. '''
        raise NotImplementedError("Vector.__size__ is not implemented")

    def has_petsc_support(self):
        ''' Returns True if the vector can be converted to a PETSc vector. '''
        raise NotImplementedError("Vector.has_petsc_support is not implemented")

    def to_petsc(self):
        ''' Returns the PETSc vector. Must only be implemented if has_petsc_support returns True. ''' 
        raise NotImplementedError("Vector.to_petsc is not implemented")


    ################# Default implementations #####################
    def __init__(self, data):
        ''' Creates a new Vector.'''
        self.data = data

    def __lmul__(self, a):
        ''' Scales the vector by a. '''
        c = self.copy()
        c.scale(a)
        return c

    __rmul__ = __lmul__

    def __add__(self, v):
        ''' Adds v to the vector. '''
        c = self.copy()
        c.axpy(1.0, v)
        return c

    def __sub__(self, v):
        ''' Adds v to the vector. '''
        c = self.copy()
        c.axpy(-1.0, v)
        return c

    def __neg__(self):
        ''' Negates the vector. '''
        c = self.copy()
        c.scale(-1)
        return c

    def __len__(self):
        ''' Returns the (local) size. '''
        return self.local_size()

    def copy(self):
        ''' Returns a deep-copy of the vector. '''
        d = self.data.__class__(self.data)
        return Vector(d)

    def assign(self, x):
        ''' Copies the values from x to the function. '''
        self.zero()
        self.axpy(1.0, x)
    



"""
The Moola optimization module
"""

__version__ = '0.1.6'
__author__  = 'Simon Funke'
__credits__ = ['Simon Funke']
__license__ = 'LGPL-3'
__maintainer__ = 'Simon Funke'
__email__ = 's.funke09@imperial.ac.uk'

from . import linalg
from . import problem
from . import linesearch
from . import algorithms
from . import misc

from .ui import *



def dcstep(stx, fx, dx, sty, fy, dy, stp, fp, dp, brackt, stpmin, stpmax):
    """

     Subroutine dcstep

     This subroutine computes a safeguarded step for a search
     procedure and updates an interval that contains a step that
     satisfies a sufficient decrease and a curvature condition.

     The parameter stx contains the step with the least function
     value. If brackt is set to True: a minimizer has
     been bracketed in an interval with endpoints stx and sty.
     The parameter stp contains the current step.
     The subroutine assumes that if brackt is set to True:

           min(stx,sty) < stp < max(stx,sty),

     and that the derivative at stx is negative in the direction
     of the step.

     The subroutine statement is

       subroutine dcstep(stx,fx,dx,sty,fy,dy,stp,fp,dp,brackt,
                         stpmin,stpmax)

     where

       stx is a double precision variable.
         On entry stx is the best step obtained so far and is an
            endpoint of the interval that contains the minimizer.
         On exit stx is the updated best step.

       fx is a double precision variable.
         On entry fx is the function at stx.
         On exit fx is the function at stx.

       dx is a double precision variable.
         On entry dx is the derivative of the function at
            stx. The derivative must be negative in the direction of
            the step, that is, dx and stp - stx must have opposite
            signs.
         On exit dx is the derivative of the function at stx.

       sty is a double precision variable.
         On entry sty is the second endpoint of the interval that
            contains the minimizer.
         On exit sty is the updated endpoint of the interval that
            contains the minimizer.

       fy is a double precision variable.
         On entry fy is the function at sty.
         On exit fy is the function at sty.

       dy is a double precision variable.
         On entry dy is the derivative of the function at sty.
         On exit dy is the derivative of the function at the exit sty.

       stp is a double precision variable.
         On entry stp is the current step. If brackt is set to True
           : on input stp must be between stx and sty.
         On exit stp is a new trial step.

       fp is a double precision variable.
         On entry fp is the function at stp
         On exit fp is unchanged.

       dp is a double precision variable.
         On entry dp is the the derivative of the function at stp.
         On exit dp is unchanged.

       brackt is an logical variable.
         On entry brackt specifies if a minimizer has been bracketed.
            Initially brackt must be set to .false.
         On exit brackt specifies if a minimizer has been bracketed.
            When a minimizer is bracketed brackt is set to True

       stpmin is a double precision variable.
         On entry stpmin is a lower bound for the step.
         On exit stpmin is unchanged.

       stpmax is a double precision variable.
         On entry stpmax is an upper bound for the step.
         On exit stpmax is unchanged.

     MINPACK-1 Project. June 1983
     Argonne National Laboratory.
     Jorge J. More' and David J. Thuente.

     MINPACK-2 Project. November 1993.
     Argonne National Laboratory and University of Minnesota.
     Brett M. Averick and Jorge J. More'.

     Converted to Python and extended, July 2013
     Imperial College London
     Simon W. Funke
    """

    from math import sqrt

    sgnd = dp*(dx/abs(dx))

    #     First case: A higher function value. The minimum is bracketed.
    #     If the cubic step is closer to stx than the quadratic step, the
    #     cubic step is taken, otherwise the average of the cubic and
    #     quadratic steps is taken.
    if fp > fx:
        theta = 3.0*(fx-fp)/(stp-stx) + dx + dp
        s = max(abs(theta), abs(dx), abs(dp))
        gamma = s*sqrt((theta/s)**2-(dx/s)*(dp/s))
        if (stp < stx):
            gamma = -gamma
        p = (gamma-dx) + theta
        q = ((gamma-dx)+gamma) + dp
        r = p/q
        stpc = stx + r*(stp-stx)
        stpq = stx + ((dx/((fx-fp)/(stp-stx)+dx))/2.0)*(stp-stx)
        if abs(stpc-stx) < abs(stpq-stx):
            stpf = stpc
        else:
            stpf = stpc + (stpq-stpc)/2.0
        brackt = True

    #     Second case: A lower function value and derivatives of opposite
    #     sign. The minimum is bracketed. If the cubic step is farther from
    #     stp than the secant step, the cubic step is taken, otherwise the
    #     secant step is taken.
    elif sgnd < 0:
        theta = 3.0*(fx-fp)/(stp-stx) + dx + dp
        s = max(abs(theta), abs(dx), abs(dp))
        gamma = s*sqrt((theta/s)**2-(dx/s)*(dp/s))

        if stp > stx: 
            gamma = -gamma
        p = (gamma-dp) + theta
        q = ((gamma-dp)+gamma) + dx
        r = p/q
        stpc = stp + r*(stx-stp)
        stpq = stp + (dp/(dp-dx))*(stx-stp)
        if abs(stpc-stp) > abs(stpq-stp):
            stpf = stpc
        else:
            stpf = stpq
        brackt = True

    #     Third case: A lower function value, derivatives of the same sign,
    #     and the magnitude of the derivative decreases.
    elif abs(dp) < abs(dx):
        #        The cubic step is computed only if the cubic tends to infinity
        #        in the direction of the step or if the minimum of the cubic
        #        is beyond stp. Otherwise the cubic step is defined to be the
        #        secant step.
        theta = 3.0*(fx-fp)/(stp-stx) + dx + dp
        s = max(abs(theta), abs(dx), abs(dp))

        #        The case gamma = 0 only arises if the cubic does not tend
        #        to infinity in the direction of the step.
        gamma = s*sqrt(max(0,(theta/s)**2-(dx/s)*(dp/s)))
        if stp > stx:
            gamma = -gamma
        p = (gamma-dp) + theta
        q = (gamma+(dx-dp)) + gamma
        r = p/q
        if r < 0 and gamma != 0:
            stpc = stp + r*(stx-stp)
        elif stp > stx:
            stpc = stpmax
        else:
            stpc = stpmin

        stpq = stp + (dp/(dp-dx))*(stx-stp)

        if brackt:
            #           A minimizer has been bracketed. If the cubic step is
            #           closer to stp than the secant step, the cubic step is
            #           taken, otherwise the secant step is taken.
            if abs(stpc-stp) < abs(stpq-stp):
                stpf = stpc
            else:
                stpf = stpq

            if stp > stx:
                stpf = min(stp+0.66*(sty-stp),stpf)
            else:
                stpf = max(stp+0.66*(sty-stp),stpf)
        else:
            #           A minimizer has not been bracketed. If the cubic step is
            #           farther from stp than the secant step, the cubic step is
            #           taken, otherwise the secant step is taken.

            if abs(stpc-stp) > abs(stpq-stp):
                stpf = stpc
            else:
                stpf = stpq
            stpf = min(stpmax,stpf)
            stpf = max(stpmin,stpf)

    #     Fourth case: A lower function value, derivatives of the same sign,
    #     and the magnitude of the derivative does not decrease. If the
    #     minimum is not bracketed, the step is either stpmin or stpmax,
    #     otherwise the cubic step is taken.
    else:
        if brackt:
            theta = 3.0*(fp-fy)/(sty-stp) + dy + dp
            s = max(abs(theta), abs(dy), abs(dp))

            gamma = s*sqrt((theta/s)**2-(dy/s)*(dp/s))
            if stp > sty:  
                gamma = -gamma
            p = (gamma-dp) + theta
            q = ((gamma-dp)+gamma) + dy
            r = p/q
            stpc = stp + r*(sty-stp)
            stpf = stpc

        elif stp > stx:
            stpf = stpmax
        else:
            stpf = stpmin

    # Update the interval which contains a minimizer.
    if fp > fx:
        sty = stp
        fy = fp
        dy = dp
    else:
        if sgnd < 0:
            sty = stx
            fy = fx
            dy = dx

        stx = stp
        fx = fp
        dx = dp

    # Compute the new step.
    stp = stpf

    return stx, fx, dx, sty, fy, dy, stp, fp, dp, brackt, stpmin, stpmax



from .optimisation_algorithm import *

class SteepestDescent(OptimisationAlgorithm):
    """
        Implements the steepest descent method.
     """
    def __init__(self, problem, initial_point=None, options={}, hooks={}, **args):
        '''
        Initialises the steepest descent algorithm.

        Valid options are:

         * options: A dictionary containing additional options for the steepest descent algorithm. Valid options are:
            - tol: Functional reduction stopping tolerance: |j - j_prev| < tol. Default: 1e-4.
            - maxiter: Maximum number of iterations before the algorithm terminates. Default: 200.
            - disp: dis/enable outputs to screen during the optimisation. Default: True
            - gtol: Gradient norm stopping tolerance: ||grad j|| < gtol.
            - line_search: defines the line search algorithm to use. Default: strong_wolfe
            - line_search_options: additional options for the line search algorithm. The specific options read the help
              for the line search algorithm.
            - an optional callback method which is called after every optimisation iteration.
         * hooks: A dictionary containing user-defined "hook" functions that are called at certain events during the optimisation.
            - before_iteration: Is called after before each iteration.
            - after_iteration: Is called after each each iteration.
          '''

        # Set the default options values
        self.problem = problem
        self.set_options(options)
        self.hooks = hooks
        self.linesearch = get_line_search_method(self.options['line_search'], self.options['line_search_options'])
        self.data = {'control'   : initial_point,
                     'iteration' : 0}

    @classmethod
    def default_options(cls):
        # this is defined as a function to prevent defaults from being changed at runtime.
        default = OptimisationAlgorithm.default_options()
        default.update(
            # generic parameters:
            {"jtol"                   : 1e-4,
             "gtol"                   : 1e-4,
             "maxiter"                :  200,
             "display"                :    2,
             "line_search"            : "strong_wolfe",
             "line_search_options"    : {"ftol": 1e-3, "gtol": 0.9, "xtol": 1e-1, "start_stp": 1},
             "record"                 : ("grad_norm", "objective"),
             })
        return default

    def __str__(self):
        s = "Steepest descent method.\n"
        s += "-"*30 + "\n"
        s += "Line search:\t\t %s\n" % self.line_search
        s += "Maximum iterations:\t %i\n" % self.maxiter
        return s

    def solve(self):
        '''
            Arguments:
             * problem: The optimisation problem.

            Return value:
              * solution: The solution to the optimisation problem
         '''
        j = None
        j_prev = None

        m = self.data['control']
        obj = self.problem.obj

        # Start the optimisation loop
        it = 0
        while True:

            grad = obj.derivative(m).primal()
            self.update({'grad_norm' : grad.norm()})
            self.display(self.iter_status, 2)

            if "before_iteration" in self.hooks:
                self.hooks["before_iteration"](j, grad)

            # Check for convergence
            if self.check_convergence() != 0:
                break

            m, alpha = self.do_linesearch(obj, m, -grad)
            j_prev, j = j, obj(m)

            # Update the iterate counter
            it += 1

            if "after_iteration" in self.hooks:
                self.hooks["after_iteration"](j, grad)

            # Print the reason for convergence
            self.update({"control"   : m,
                         "iteration" : it,
                         "grad_norm" : grad.norm(),
                         "objective" : j})
            self.record_progress()
        return self.data



''' Computes the Cauchy point as descirbed in Wright 1999, section 16.6 '''
import numpy as np
from .infinity import inf

def P(x, l, u):
    '''
    Projects x onto the bounds l and u:

                  / l_i if x_i < l_i,
    P(x, l, u) := | x_i if x_i in [l_i, u_i],
                  \ u_i if x_i > u_i.
    '''
    for i in range(len(x)):
        if x[i] < l[i]:
            x[i] = l[i] 
        elif x[i] > u[i]:
            x[i] = u[i]

def x(x0, g, t, l, u):
    ''' Returns P(x0 - g*t) '''
    x = x0.__class__(x0)
    x.axpy(-t, g)
    P(x, l, u)
    return x

def compute_cauchy_point(G, d, x0, l, u):
    ''' Computes the Cauchy point x_c := x0 - tg, 

    where t is the first local minimizer of a piecewise quadratic function q, i.e.:

    min_t q(x(t)) := 1/2x(t)^TGx(t) + x(t)^Td,

    with

    g := grad q(x0) = Gx0 + d,
    
    x(t) := P(x^0 - tg, l, u),

    and P the projection operator onto the bounds:

                  / l_i if x_i < l_i,
    P(x, l, u) := | x_i if x_i in [l_i, u_i],
                  \ u_i if x_i > u_i.

    The notation follows Wright 1999, section 16.6
    '''

    # Compute the gradient of q at x0 
    g = G(x0)
    g.axpy(1, d)

    # Compute values of t when kinks in x(t) occur
    tbar = g.__class__(g)
    for i in range(len(tbar)):
        if g[i] < 0 and u[i] < inf:
            tbar[i] = (x0[i] - u[i])/g[i]
        elif g[i] > 0 and l[i] > -inf:
            tbar[i] = (x0[i] - l[i])/g[i]
        else:
            tbar[i] = inf

    # Sort these t values in increasing order
    tbar_arr = tbar.array()
    sort_index = np.argsort(tbar_arr) # 1st element: index to smallest element, ... 
    t = lambda j: tbar_arr[sort_index[j]] if j >= 0 else 0. # A helper function to access the t values in order defined for -1 <= j < len(sort_index).

    # Loop over all the tbar's in order until we find the local minimum
    xc = None
    for j in range(len(sort_index)):
        # Update the search direction 
        # TODO Instead of recomputing p it could be updated 
        p = g.__class__(g)
        p.scale(-1)
        for i in range(len(p)):
            if not t(j-1) < tbar[i]:
                p[i] = 0 

        xt = x(x0, g, t(j-1), l, u)

        # Check if the minimizer lies within t(j-1) <= t < t(j)
        Gp = G(p)
        df = d.inner(p) + xt.inner(Gp)
        ddf = p.inner(Gp) 
        if ddf > 0 and 0 <= -df/ddf <= t(j) - t(j-1):
            # Local minimizer found
            t_opt = t(j-1) - df/ddf
            xc = x(x0, g, t_opt, l, u)
            break
        elif df > 0:
            # Local minimizer at t(j-1)
            xc = xt 
            break

    if not xc:
        # No mininizer was found in 0 <= t < t(len(sort_index)) 
        xc = x(x0, g, t(len(sort_index)-1), l, u)

    return xc
    




from .optimisation_algorithm import *
from .bfgs import LimitedMemoryInverseHessian, LinearOperator
from numpy import sqrt
from IPython import embed

def dual_to_primal(x):
    return x.primal()

class HybridCG(OptimisationAlgorithm):
    ''' 
    A Newton-CG method that falls back to L-BFGS when the curvature is non-positive.
    '''
    __name__ = 'NewtonCG'
    def __init__(self, problem, initial_point = None, Hinit=LinearOperator(dual_to_primal), options = {}):
        '''
        Initialises the Hybrid CG method. The valid options are:
         * options: A dictionary containing additional options for the steepest descent algorithm. Valid options are:
            - tol: Not supported yet - must be None. 
            - maxiter: Maximum number of iterations before the algorithm terminates. Default: 200. 
            - disp: dis/enable outputs to screen during the optimisation. Default: True
            - gtol: Gradient norm stopping tolerance: ||grad j|| < gtol.
            - line_search: defines the line search algorithm to use. Default: strong_wolfe
            - line_search_options: additional options for the line search algorithm. The specific options read the help 
              for the line search algorithm.
            - an optional callback method which is called after every optimisation iteration.
          '''

        # Set the default options values
        self.problem = problem
        self.set_options(options)
        self.linesearch = get_line_search_method(self.options['line_search'], self.options['line_search_options'])
        self.data = {'control'   : initial_point,
                     'iteration' : 0,
                     'lbfgs'     : LimitedMemoryInverseHessian(Hinit, self.options['mem_lim']) }
    
    def __str__(self):
        s = "Hybrid L-FBGS / CG method.\n"
        s += "-"*30 + "\n"
        s += "Line search:\t\t %s\n" % self.options['line_search']
        s += "Maximum iterations:\t %i\n" % self.options['maxiter']
        return s
    
    # set default parameters
    
    @classmethod
    def default_options(cls):
        # this is defined as a function to prevent defaults from being changed at runtime.
        default = OptimisationAlgorithm.default_options()
        default.update(
            # generic parameters:
            {"jtol"                   : None,
             "gtol"                   : 1e-4,
             "maxiter"                :  200,
             "display"                :    2,
             "line_search"            : "strong_wolfe",
             "line_search_options"    : {},
             "callback"               : None,
             "record"                 : ("grad_norm", "objective"),

             # method specific parameters:
             "initial_bfgs_iterations": 0,
             "ncg_reltol"             :  .5,
             "ncg_maxiter"            : 200,
             "ncg_hesstol"            : "default",
             "mem_lim"                : 5,
             "Hinit"                  : "default",
             })
        return default

    def solve(self):
        '''
            Arguments:
             * problem: The optimisation problem.

            Return value:
              * solution: The solution to the optimisation problem 
         '''
        self.display( self.__str__(), 1)

        objective = self.problem.obj
        options = self.options
        
        
        B = self.data['lbfgs']
        x = self.data['control']
        i = self.data['iteration']

        # compute initial objective and gradient
        J = objective(x)
        r = objective.derivative(x)  # initial residual ( with dk = 0)
        r.scale(-1.)
        self.update({'objective' : J,
                     'grad_norm' : r.primal_norm()})
        self.record_progress()
        
        if options['ncg_hesstol'] == "default":
            import numpy
            eps = numpy.finfo(numpy.float64).eps
            ncg_hesstol = eps*numpy.sqrt(len(x))
        else:
            ncg_hesstol = options['ncg_hesstol']
        
        # Start the optimisation loop
        while self.check_convergence() == 0:
            self.display(self.iter_status, 2)
            r0 = r.copy()
            p = Br = (B * r) # mapping residual to primal space
            d = p.copy().zero()
            rBr = r.apply(Br)
            H = objective.hessian(x)
            
            
            # CG iterations
            cg_tol =  min(options['ncg_reltol']**2, sqrt(rBr))*rBr 
            cg_iter  = 0
            cg_break = 0
            
            while cg_iter < options['ncg_maxiter'] and rBr >= cg_tol:
                if i < options['initial_bfgs_iterations']:
                    d = Br
                    break
                Hp  = H(p)
                pHp = Hp.apply(p)
                
                self.display('cg_iter = {}\tcurve = {}\thesstol = {}'.format(cg_iter, pHp, ncg_hesstol), 3)
                if pHp < 0:
                    #print 'TEST: not descent direction'
                    if cg_iter == 0:
                        # Fall back to L-BFGS update
                        d = Br
                        pass
                    # otherwise use the last computed pk
                    break
                        
                if 0 <= pHp < ncg_hesstol:
                    #cg_break = 2
                    if cg_iter == 0:
                        d = Br
                    # try to use what we have
                    try:
                        self.do_linesearch(objective, x, d) #TODO: fix this hack
                        #print 'TEST: below curvature treshold'
                        break
                    except:
                        pass
                # Standard CG iterations
                alpha = rBr / pHp
                d.axpy(alpha, p)            # update cg iterate
                r.axpy(-alpha, Hp)          # update residual

                Br = B*r
                t = r.apply(Br)
                rBr, beta = t, t / rBr, 

                p.scale(beta)
                p.axpy(1., Br)
                
                cg_iter +=1

            
            # do a line search and update
            x, a = self.do_linesearch(objective, x, d)
            d.scale(a)
            
            
            J, oldJ = objective(x), J

            # evaluate gradient at the new point
            r, r0 = objective.derivative(x), r0
            r.scale(-1)
            y = r0 - r
            
            # update the approximate Hessian
            
            B.update(y, d)
            i += 1

            # store current iteration variables
            self.update({'iteration' : i,
                         'control'   : x,
                         'grad_norm' : r.primal_norm(),
                         'delta_J'   : oldJ-J,
                         'objective' : J,
                         'lbfgs'     : B })
            self.record_progress()
        self.display(self.convergence_status, 1)
        self.display(self.iter_status, 1)
        return self.data
    
    
    
                
 



"""
Line search methods for the Moola optimisation module.
"""

__author__  = 'Simon Funke'
__credits__ = ['Simon Funke']
__license__ = 'LGPL-3'
__maintainer__ = 'Simon Funke'
__email__ = 's.funke09@imperial.ac.uk'

from .armijo import ArmijoLineSearch
from .strong_wolfe import StrongWolfeLineSearch
from .fixed import FixedLineSearch
from .hager_zhang import HagerZhangLineSearch



from .dolfin_vector import DolfinVector, DolfinPrimalVector, DolfinDualVector
from moola.linalg import Vector
from moola.misc import events
from math import sqrt
from numpy import array, zeros, ndarray, eye

class RieszMapSet(object):

    def __init__(self, inner_product="l2", map_operator=None, inverse = "default"):
        
        if inner_product == "l2":
            self.riesz_map = 1
            self.riesz_inv = 1
            
        elif inner_product == "custom":
            from numpy import ndarray, equal
            from scipy.sparse.linalg import LinearOperator, aslinearoperator
            if not isinstance(map_operator, (ndarray, LinearOperator)) or not equal(*map_operator.shape):
                raise TypeError("only square numpy arrays are currently supported")
            self.riesz_map = aslinearoperator(map_operator)
            
            if inverse in ("default", "lu") and isinstance(map_operator, ndarray):
                from numpy.linalg import inv
                self.riesz_inv = aslinearoperator(inv(map_operator))
            
            else:
                self.riesz_inv = inverse
        
        self.inner_product = inner_product
                
    def primal_map(self, x, b):
        if isinstance(self.riesz_inv, ndarray):
            b[:] = self.riesz_inv.dot(x.vector_list)
        else:
            b[:] = self.riesz_inv * x.vector_list

    def dual_map(self, x):
        if isinstance(self.riesz_map, ndarray):
            b[:] = self.riesz_map.dot(x.vector_list)
        else:
            b[:] = self.riesz_map * x.vector_list

class IdentityMapSet(RieszMapSet):
    def __init__(self):
        RieszMapSet.__init__(self, inner_product = "l2")

class DolfinVectorSet(Vector):

    def __init__(self, vector_list, riesz_map = None):
        '''An implementation for set of vectors based on FEniCS data types.
        Currently supports only simple Riesz maps of numpy.ndarry type.

        Args:
            vector_list (list): A list with moola.DolfinVector
            
            riesz_map (array): An operator to be applied to the vector
            of controls prior to the individual control Riesz maps.

        '''

        for vec in vector_list:
            if not isinstance(vec, DolfinVector):
                raise ValueError("vector_list must be a list of DolfinVectors")
        self.vector_list = zeros(len(vector_list), dtype = object)
        self.vector_list[:] = vector_list
        
        if riesz_map == None:
            riesz_map = RieszMapSet("l2")
            
        self.riesz_map = riesz_map

    def __len__(self):
        return len(self.vector_list)

    def __getitem__(self, index):
        ''' Returns the subvector with given index. '''
        return self.vector_list[index]

    def __setitem__(self, index, value):
        ''' Sets the subvector with the given index. '''
        self.vector_list[index] = value

    @property
    def data(self):
        return [vec.data for vec in self.vector_list]

    def array(self):
        ''' Returns the vector as a numpy.array object. If local=False, the
        global array must be returned in a distributed environment. '''
        #return self.data.vector().array()
        raise NotImplementedError

    def scale(self, s):
        [vec.scale(s) for vec in self.vector_list]

    def __hash__(self):
        ''' Returns a hash of the vector '''
        hashes = tuple([hash(vec) for vec in self.vector_list])
        return hash(hashes)

    def axpy(self, a, x):
        ''' Adds a*x to the function. '''
        assert self.__class__ == x.__class__
        return [vec.axpy(a, xx) for vec, xx in zip(self.vector_list, x.vector_list)]


    def zero(self):
        ''' Zeros the function. '''
        [vec.zero() for vec in self.vector_list]
        return self

    def local_size(self):
        ''' Returns the (local) size of the vector. '''
        #return self.data.vector().local_size()
        raise NotImplementedError

    def size(self):
        ''' Returns the (gobal) size of the vector. '''
        return sum([vec.size() for vec in self.vector_list])

    def copy(self):
        vector_list_cpy = [vec.copy() for vec in self.vector_list]
        if hasattr(self, "riesz_map"):
            return self.__class__(vector_list_cpy, riesz_map = self.riesz_map)
        else:
            return self.__class__(vector_list_cpy)


class DolfinPrimalVectorSet(DolfinVectorSet):
    """ A class for representing primal vectors. """
    def __init__(self, vector_list, riesz_map = None):
        for i, v in enumerate(vector_list):
            if not isinstance(v, DolfinPrimalVector):
                raise TypeError("Vector with index {} is not a DolfinPrimalVector.".format(i))
        DolfinVectorSet.__init__(self, vector_list, riesz_map = riesz_map)

    def dual(self):
        """ Returns the dual representation. """
        if self.riesz_map.inner_product == "l2":
            return DolfinDualVectorSet([vec.dual() for vec in self.vector_list],
                                       riesz_map = self.riesz_map)
        else:
            return DolfinDualVectorSet([vec.dual() for vec in self.riesz_map.riesz_map * self.vector_list],
                                       riesz_map = self.riesz_map)
            

    def inner(self, vec):
        """ Computes the inner product with vec. """
        assert isinstance(vec, DolfinPrimalVectorSet)
        events.increment("Inner product")

        return sum([ss.inner(vec) for ss, vec in zip(self.vector_list, vec.vector_list)])

    def norm(self):
        """ Computes the vector norm induced by the inner product. """

        return sqrt(self.inner(self))
    primal_norm = norm


class DolfinDualVectorSet(DolfinVectorSet):
    """ A class for representing dual vectors. """
    def __init__(self, vector_list, riesz_map = None):
        for i, v in enumerate(vector_list):
            if not isinstance(v, DolfinDualVector):
                raise TypeError("Vector with index {} is not a DolfinDualVector.".format(i))
        DolfinVectorSet.__init__(self, vector_list, riesz_map)

    def apply(self, primal):
        """ Applies the dual vector to a primal vector. """
        assert isinstance(primal, DolfinPrimalVectorSet)
        return sum([ss.apply(vec) for ss, vec in zip(self.vector_list, primal.vector_list)])

    def primal(self):
        """ Returns the primal representation. """
        #events.increment("Dual -> primal map")
        if self.riesz_map.inner_product == "l2":
            return DolfinPrimalVectorSet([vec.primal() for vec in self.vector_list], 
                                         riesz_map = self.riesz_map)
        else:
            primal_vecs = zeros(len(self), dtype = "object")
            primal_vecs[:] = [v.primal() for v in self.vector_list]
            return DolfinPrimalVectorSet(self.riesz_map.riesz_inv * primal_vecs,
                                         riesz_map = self.riesz_map)
            

    def primal_norm(self):
        """ Computes the norm of the primal representation. """

        return sqrt(self.apply(self.primal()))

DolfinLinearFunctional = DolfinDualVector



from .line_search import LineSearch
from .dcsrch import dcsrch
from numpy import zeros
from logging import error

class StrongWolfeLineSearch(LineSearch):
    def __init__(self, ftol=1e-4, gtol=0.9, xtol=1e-1, start_stp=1.0, stpmin = None, stpmax="automatic", verify=False, ignore_warnings=False):
        '''
        This class implements a line search algorithm whose steps
        satisfy the strong Wolfe conditions (i.e. they satisfies a
        sufficient decrease condition and a curvature condition).

        The algorithm is designed to find a step 'stp' that satisfies
        the sufficient decrease condition

               f(stp) <= f(0) + ftol*stp*f'(0),

        and the curvature condition

               abs(f'(stp)) <= gtol*abs(f'(0)).

        If ftol is less than gtol and if, for example, the function
        is bounded below, then there is always a step which satisfies
        both conditions.

        If no step can be found that satisfies both conditions, then
        the algorithm stops with a warning. In this case stp only
        satisfies the sufficient decrease condition.

        The function arguments are:

           ftol           | a nonnegative tolerance for the sufficient decrease condition.
           gtol           | a nonnegative tolerance for the curvature condition.
           xtol           | a nonnegative relative tolerance for an acceptable step.
           start_stp      | a guess for an initial step size.
           stpmin         | a nonnegative lower bound for the step.
           stpmax         | a nonnegative upper bound for the step.
           verify         | if True, the step is compared to the Fortran implementation
                          | (note: this assumes that the Python module dcsrch_fortran is compiled
                          |  and in the PYTHONPATH)
          ignore_warnings | Continue if the line search ends with a warnings (e.g. stp = stpmax).
                          | Default: True

        References:
         Mor'e, J.J., and Thuente, D.J., 1992, Line search algorithms with guaranteed
              sufficient decrease: Preprint MCS-P330-1092, Argonne National Laboratory.
         Averick, B.M., and Mor'e, J.J., 1993, FORTRAN subroutines dcstep and dcsrch
              from MINPACK-2, 1993, Argonne National Laboratory and University of Minnesota.
        '''

        self.ftol            = ftol
        self.gtol            = gtol
        self.xtol            = xtol
        self.start_stp       = start_stp
        self.stpmin          = stpmin
        self.stpmax          = stpmax
        self.verify          = verify
        self.ignore_warnings = ignore_warnings

    def search(self, phi, phi_dphi, phi_dphi0):
        ''' Performs the line search on the function phi.

            phi must be a function [0, oo] -> R.
            phi_dphi must evaluate phi and its derivative, and
            must be a function [0, oo] -> (R, R).

            The return value is a step that satisfies the strong Wolfe condition.
        '''

        # Set up the variables for dscrch
        isave = zeros(3)
        dsave = zeros(14)
        task = "START"

        f, g = phi_dphi0

        # Compute an estimate for the maximum step size
        if not self.stpmin:
            self.stpmin = 0.
        if self.stpmax == "automatic":
            stpmax = max(4*min(self.start_stp, 1.0), 0.1*f/(-g*self.ftol))
            print("-g", -g)
        else:
            stpmax = self.stpmax

        stp = min(self.start_stp, stpmax)

        while True:
            stp, task, isave, dsave = self.__csrch__(f, g, stp, task, isave, dsave, stpmax)

            if task in ("START", "FG"):
                f, g = phi_dphi(stp)
            else:
                break

        if "Error" in task:
            raise RuntimeError(task)
        elif "Warning" in task:
            if not self.ignore_warnings:
                raise Warning(task)
            else:
                print("Warning in line search: %s." % task.replace("Warning: ", ""))
                return stp
        else:
            assert task=="Convergence" or ("Warning" in task and self.ignore_warnings)

            if self.verify:
                # Recompute the step with the Fortran implementation and compare
                stp_fort = self.search_fortran(phi, phi_dphi, stpmax)
                if stp_fort is not None and stp_fort != stp:
                    raise RuntimeError("The line search verification failed!")

            return stp

    def search_fortran(self, phi, phi_dphi, stpmax):
        ''' Performs the line search on the function phi using the Fortran implementation
            of the line search algorithm.

            phi must be a function [0, oo] -> R.
            phi_dphi must evaluate phi and its derivative, and
            must be a function [0, oo] -> (R, R).

            The return value is a step that satisfies the strong Wolfe condition.
        '''

        try:
            import pyswolfe
            dphi = lambda x: phi_dphi(x)[1]
            ls_fort = pyswolfe.StrongWolfeLineSearch(phi(0), dphi(0), 1.0, phi, dphi, gtol=self.gtol,
                                                     xtol=self.xtol, ftol=self.ftol, stp=self.start_stp,
                                                     stpmin=self.stpmin, stpmax=stpmax)
            ls_fort.search()
            return ls_fort.stp

        except ImportError:
            error("The line search could not be verified. Did you compile the pyswolfe Fortran module?")

    def __csrch__(self, f, g, stp, task, isave, dsave, stpmax):
        stp, task, isave, dsave = dcsrch(stp, f, g, self.ftol, self.gtol, self.xtol, task, self.stpmin, stpmax, isave, dsave)
        return stp, task, isave, dsave




class Functional(object):

    def __call__(self, x):
        ''' Evaluates the object functional at x. '''

        raise NotImplementedError("Functional.__call__ is not implemented.") 

    def gradient(self, x):
        ''' Evaluates the gradient of the object functional at x. 
            The returned gradient is the Riez representation of the 
            derivative, i.e. is a vector in the same space.
        '''

        return self.derivative(x).primal()

    def derivative(self, x):
        ''' Returns the derivative operator at x. '''

        raise NotImplementedError("Functional.derivative is not implemented.") 


    def Hessian(self, x):
        ''' Returns the Hessian of the object functional at x as a function. '''
        raise NotImplementedError("Functional.Hessian is not implemented.") 



from .steepest_descent import SteepestDescent
from .nonlinear_cg import NonLinearCG
from .bfgs import BFGS
from .newton_cg import NewtonCG
from .newton_cg_trust_region import TrustRegionNewtonCG
from .hybrid_cg import HybridCG



from .dolfin_vector import DolfinPrimalVector, DolfinDualVector
from .dolfin_vector_set import DolfinDualVectorSet
from .numpy_vector import NumpyPrimalVector, NumpyDualVector


def convert_to_moola_dual_vector(x, y):
    """Convert `x` to a moola dual vector with primal vector `y`.

    """
    if isinstance(y, DolfinPrimalVector):
        r = DolfinDualVector(x, riesz_map=y.riesz_map)
    elif isinstance(y, NumpyPrimalVector):
        r = NumpyDualVector(x)
    else:
        r = DolfinDualVectorSet(
            [DolfinDualVector(xi, riesz_map=yi.riesz_map) for (xi, yi) in zip(x, y.vector_list)],
            riesz_map=y.riesz_map)

    return r




#!/usr/bin/env python

from setuptools import setup, find_packages

setup(name='moola',
      version='0.1.6',
      description='Moola optimisation package',
      author='Simon Funke',
      author_email='simon@simula.no',
      url='https://github.com/funsim/moola',
      packages=find_packages(),
      classifiers=[
            'Programming Language :: Python',
            'Programming Language :: Python :: 2',
            'Operating System :: OS Independent',
            'Environment :: Console',
            'Intended Audience :: Science/Research',
            'Topic :: Scientific/Engineering :: Mathematics'
      ]
)



from moola.linalg import Vector
from moola.misc import events
from math import sqrt
import numpy as np

class NumpyVector(Vector):
    ''' An implementation for vectors based on numpy arrays. '''

    def __init__(self, data):
        self.data = np.array(data, dtype=np.float64)

    def __getitem__(self, index):
        ''' Returns the value of the (local) index. '''
        return self.data[index]

    def __setitem__(self, index, value):
        ''' Sets the value of the (local) index. '''
        self.data[index] = value

    def array(self):
        ''' Returns the vector as a numpy.array object. If local=False, the 
        global array must be returned in a distributed environment. '''
        return self.data

    def scale(self, s):
        self.data *= s

    def axpy(self, a, x):
        ''' Adds a*x to the function. '''
        self.data += a*x.data

    def local_size(self):
        ''' Returns the (local) size of the vector. '''
        return len(self.data)

    def zero(self):
        ''' Zeros the vector. '''
        self.data[:] = 0
        return self

    def size(self):
        ''' Returns the (global) size of the vector. '''
        return len(self.data)

    def copy(self):
        return self.__class__(self.data.copy())


class NumpyPrimalVector(NumpyVector):
    """ A class for representing primal vectors. """

    def dual(self):
        """ Returns the dual representation. """

        events.increment("Primal -> dual map")
        return NumpyDualVector(self.data.copy())

    def inner(self, vec):
        """ Computes the inner product with vec. """
        assert isinstance(vec, NumpyPrimalVector)
        events.increment("Inner product")

        return float(np.dot(self.data, vec.data))

    def norm(self):
        """ Computes the vector norm induced by the inner product. """

        return sqrt(self.inner(self))
    primal_norm = norm


class NumpyDualVector(NumpyVector):
    """ A class for representing dual vectors. """

    def apply(self, primal):
        """ Applies the dual vector to a primal vector. """
        assert isinstance(primal, NumpyPrimalVector)
        return float(np.dot(self.data, primal.data))
    
    def primal(self):
        """ Returns the primal representation. """
        events.increment("Dual -> primal map")

        return NumpyPrimalVector(self.data.copy())

    def primal_norm(self):
        """ Computes the norm of the primal representation. """

        return sqrt(self.apply(self.primal()))


NumpyLinearFunctional = NumpyDualVector



"""
PyMSWolfe: Jorge Nocedal's modified More and Thuente linesearch
guaranteeing satisfaction of the strong Wolfe conditions.
"""

import numpy
from nlpy.optimize.ls import _pymcsrch

class StrongWolfeLineSearch:
    """
    A general-purpose linesearch procedure enforcing the strong
    Wolfe conditions

    f(x+td) <= f(x) + ftol * t * <g(x),d>    (Armijo condition)

    | <g(x+td),d> | <= gtol * | <g(x),d> |   (curvature condition)

    This is a Python interface to Jorge Nocedal's modification of
    the More and Thuente linesearch. Usage of this class is slightly
    different from the original More and Thuente linesearch.

    Instantiate as follows

    SWLS = StrongWolfeLineSearch(f, x, g, d, obj, grad)

    where

    * f     is the objective value at the current iterate x
    * x     is the current iterate
    * g     is the objective gradient at the current iterate x
    * d     is the current search direction
    * obj   is a scalar function used to evaluate the value of
            the objective at a given point
    * grad  is a scalar function used to evaluate the gradient
            of the objective at a given point.

    :keywords:

        :ftol:    the constant used in the Armijo condition (1e-4)
        :gtol:    the constant used in the curvature condition (0.9)
        :xtol:    a minimal relative step bracket length (1e-16)
        :stp:     an initial step value (1.0)
        :stpmin:  the initial lower bound of the bracket (1e-20)
        :stpmax:  the initial upper bound of the bracket (1e+20)
        :maxfev:  the maximum number of function evaluations permitted (20)

    To ensure existence of a step satisfying the strong Wolfe
    conditions, d should be a descent direction for f at x and
    ftol <= gtol.

    The final value of the step will be held in SWLS.stp

    After the search, SWLS.armijo will be set to True if the step
    computed satisfies the Armijo condition and SWLS.curvature will
    be set to True if the step satisfies the curvature condition.
    """
    def __init__(self, f, x, g, d, obj, grad, **kwargs):

        # Mandatory arguments
        self.f = f          # Function value f(xk)
        self.x = x.copy()   # xk
        self.g = g.copy()   # Gradient of f at xk
        self.d = d          # Direction along which to search
        self.n = self.g.shape[0]

        self.obj  = obj   # To evaluate function value
        self.grad = grad  # To evaluate function gradient

        # Optional arguments
        self.ftol   = kwargs.get('ftol', 1.0e-4)
        self.gtol   = kwargs.get('gtol', 0.9)
        self.xtol   = kwargs.get('xtol', 1.0e-16)
        self.stp    = kwargs.get('stp', 1.0)
        self.stpmin = kwargs.get('stpmin', 1.0e-20)
        self.stpmax = kwargs.get('stpmax', 1.0e+20)
        self.maxfev = kwargs.get('maxfev', 20)

        # Initialize context object
        self.context = _pymcsrch.Init(self.n,
                                       self.ftol,
                                       self.gtol,
                                       self.xtol,
                                       self.stp,
                                       self.stpmin,
                                       self.stpmax,
                                       self.maxfev,
                                       self.d)

        self.armijo = False
        self.curvature = False
        self.info = None

    def search(self):

        self.stp, info = self.context.mcsrch(self.f,self.x,self.g)

        while info == -1:
            self.f = self.obj(self.x)
            self.g = self.grad(self.x)
            self.stp, info = self.context.mcsrch(self.f,self.x,self.g)

        if info == 1:     # Strong Wolfe conditions satisfied
            self.armijo = True
            self.curvature = True

        if info == 0:
            print(' linesearch returned; incorrect input values:')
            print('    stpmin = ', self.stpmin)
            print('    stpmax = ', self.stpmax)
            print('    ftol   = ', self.ftol)
            print('    gtol   = ', self.gtol)
            print('    xtol   = ', self.xtol)
            print('    n      = ', self.n)
            print('    stp    = ', self.stp)
            print('    maxfev = ', self.maxfev)
        self.info = info
        return


if __name__ == '__main__':

    import amplpy
    import numpy
    from math import sqrt
    import sys

    nlp = amplpy.AmplModel(sys.argv[1])
    f = nlp.obj(nlp.x0)
    g = nlp.grad(nlp.x0)
    d = -g
    SWLS = StrongWolfeLineSearch(f,
                                  nlp.x0,
                                  g,
                                  d,
                                  lambda z: nlp.obj(z),
                                  lambda z: nlp.grad(z),
                                  stp = 1.0/sqrt(numpy.dot(g,g)))
    print(' Before search')
    print('   f = ', f)
    print('   stpmax = ', SWLS.stpmax)
    SWLS.search()
    print(' After search')
    print('   f = ', SWLS.f)
    print('   step length = ', SWLS.stp)
    nlp.close()



from moola.linesearch import FixedLineSearch
from moola.linesearch import ArmijoLineSearch
from moola.linesearch import StrongWolfeLineSearch
from moola.linesearch import HagerZhangLineSearch
from moola.problem import Solution

class OptimisationAlgorithm(object):
    '''
    An abstract implementation of an optimisation algorithm.
    All implementations of optimisation algorithms should use this is a base class.
    '''
    def __init__(self, tol, options={}, **args):
        ''' Initialises the optmization algorithm. '''
        raise NotImplementedError('OptimisationAlgorithm.__init__ is not implemented')

    def __str__(self):
        ''' Prints out a description of the algorithm settings. '''
        raise NotImplementedError('OptimisationAlgorithm.__str__ is not implemented')


    @classmethod
    def default_options(cls):
        # this should set any parameter that all algorithms needs to know.
        default = {'display': 10,
                   'maxiter': 100,}
        return default

    def set_options(self, user_options):
        # Update options with provided dictionary.
        if not isinstance(user_options, dict):
            raise TypeError("Options have to be set with a dictionary object.")
        if hasattr(self,  'options'):
            options = self.options
        else:
            options = self.default_options()
        for key, val in user_options.items():
            if key not in options:
                raise KeyError("'{}' not a valid setting for {}".format(key, self.__class__.__name__))
            # TODO: check also that the provided value is admissible.
            options[key] = val
        self.options = options
        return options

    def display(self, text, level):
        '''
        Function for providing information to the user.
        Only prints text if level is not smaller than the treshold setting.

        Tentative level structuring:
          0: no output (user may have provided a callback function to store or print data)
          1: start / end of optimization loop
          2: general information for each optimization loop, e.g. iterations number, gradient norm etc
          3: specific information for each optimization loop, e.g. inner iteration loop information
           :
           :
         10: debug
        '''
        if level <= self.options['display']:
            print(text)

    def do_linesearch(self, obj, m, s):
        ''' Performs a linesearch on obj starting from m in direction s. '''

        m_new = m.copy()

        def update_m_new(alpha):
            if update_m_new.alpha_new != alpha:
                m_new.assign(m)
                m_new.axpy(alpha, s)
                update_m_new.alpha_new = alpha
        update_m_new.alpha_new = 0

        # Define the real-valued functions in the s-direction
        def phi(alpha):
            update_m_new(alpha)

            return obj(m_new)

        def phi_dphi(alpha):
            update_m_new(alpha)

            p = obj(m_new)
            djs = obj.derivative(m_new).apply(s)
            return p, djs

        #get values at current point
        phi_dphi0 = phi_dphi(0)

        # Perform the line search
        alpha = self.linesearch.search(phi, phi_dphi, phi_dphi0)

        update_m_new(alpha)
        return m_new, float(alpha)

    def check_convergence(self):
        '''
        Unified convergence test for the implemented optimization algorithms.
        '''
        data = self.data
        options = self.options
        status = 0
        if data['iteration'] >= self.options['maxiter']:
            status = -1
        if 'gtol' in options and options['gtol'] != None and 'grad_norm' in data:
            if data['grad_norm'] < options['gtol']:
                status = 1
        if 'rgtol' in options and options['rgtol'] != None and 'grad_norm' in data and 'initial_grad_norm' in data:
            if data['grad_norm'] < options['rgtol'] * data['initial_grad_norm']:
                status = 1
        if 'jtol' in options and options['jtol'] != None and 'delta_J' in data:
            if data['delta_J'] < options['jtol']:
                status = 2
        if 'rjtol' in options and options['rjtol'] != None and 'delta_J' in data and 'initial_J' in data:
            if data['delta_J'] < options['rjtol'] * data['initial_J']:
                status = 2
        # TODO: implement more tests
        data['status'] = status
        return status

    @property
    def iter_status(self):
        keys = ['iteration', 'objective', 'grad_norm', 'delta_J', 'delta_x']
        return '\t'.join(['{} = {}:'.format(k, self.data[k]) for k in keys if k in self.data])

    @property
    def convergence_status(self):
        reasons = {-1: '\nMaximum number of iterations reached.\n',
                    2: '\nTolerance reached: delta_j < jtol.\n',
                    1: '\nTolerance reached: grad_norm < gtol.\n',
                   -4: 'Linesearch failed.',
                   -5: 'Algorithm breakdown.'}
        return reasons[self.data['status']]


    def solve(self):
        ''' Solves the optimisation problem.
            Arguments:
             * problem: The optimisation problem.

            Return value:
              * solution: The solution to the optimisation problem
        '''
        raise NotImplementedError('OptimisationAlgorithm.solve is not implemented')

    def update(self, d=None, **args):
        if not hasattr(self, 'data'):
            self.data = {}
        if d is not None and hasattr(d, 'keys'):
            self.data.update(d)
        else:
            self.data.update(**args)
    def record_progress(self, **args):
        if args == {}:
            args = self.options['record']
        if not hasattr(self, 'history'):
            # then initialize it
            self.history = {arg: [] for arg in args}
        for arg in args:
            self.history[arg].append(self.data[arg])



def get_line_search_method(line_search, line_search_options):
    ''' Takes a name of a line search method and returns its Python implementation as a function. '''
    if line_search == "strong_wolfe":
        ls = StrongWolfeLineSearch(**line_search_options)
    elif line_search == "approximate_wolfe":
        ls = HagerZhangLineSearch(**line_search_options)
    elif line_search == "armijo":
        ls = ArmijoLineSearch(**line_search_options)
    elif line_search == "fixed":
        ls = FixedLineSearch(**line_search_options)
    else:
        raise ValueError("Unknown line search specified. Valid values are 'armijo', 'strong_wolfe', 'approximate_wolfe' and 'fixed'.")

    return ls


